import sys
import platform
import atexit
from pathlib import Path
from PyQt6.QtWidgets import QApplication, QToolTip, QMessageBox
from PyQt6.QtGui import QFont, QFontDatabase
from PyQt6.QtCore import Qt
from ui.main_window import MainWindow
from models import PluginStore

if __name__ == "__main__":
    # Create application
    app = QApplication(sys.argv)
    app.setApplicationName("NotePlan Plugin Launcher")
    
    # Set global tooltip settings
    app.setStyleSheet("""
        QToolTip { 
            font-size: 12px; 
            max-width: 500px; 
            opacity: 225; 
            background-color: #f8f8f8; 
            color: #333333;
            border: 1px solid #bbbbbb;
            padding: 5px;
            border-radius: 3px;
        }
    """)
    
    # Set high DPI support
    # In PyQt6, high DPI is enabled by default
    
    # Use Fusion style for a consistent look across platforms
    app.setStyle("Fusion")
    
    # Load and set system font based on platform
    if platform.system() == "Darwin":  # macOS
        default_font = QFont("SF Pro Display", 10)
    elif platform.system() == "Windows":
        default_font = QFont("Segoe UI", 10)
    else:  # Linux and others
        default_font = QFont("Ubuntu", 10)
    
    app.setFont(default_font)
    
    # Add debug function for finding NotePlan plugins path
    def find_noteplan_plugins_paths():
        """Find potential NotePlan plugin paths for debugging"""
        from pathlib import Path
        home = Path.home()
        
        # Define potential paths where plugins could be located
        potential_paths = [
            # Sandboxed Setapp version
            home / "Library/Containers/co.noteplan.NotePlan-setapp/Data/Library/Application Support/co.noteplan.NotePlan-setapp/Plugins",
            
            # Normal sandboxed version
            home / "Library/Containers/co.noteplan.NotePlan/Data/Library/Application Support/co.noteplan.NotePlan/Plugins",
            
            # Direct non-sandboxed version 
            home / "Library/Application Support/co.noteplan.NotePlan/Plugins",
            
            # Older version
            home / "Library/Application Support/NotePlan/Plugins",
        ]
        
        # Check containers directory for any other NotePlan variants
        containers_path = home / "Library" / "Containers"
        if containers_path.exists():
            try:
                for container in containers_path.iterdir():
                    if container.name.startswith("co.noteplan.NotePlan") and container.is_dir():
                        container_path = container / "Data" / "Library" / "Application Support" / container.name / "Plugins"
                        # Only add if not already in our list
                        if container_path not in potential_paths:
                            potential_paths.append(container_path)
            except Exception as e:
                print(f"Error checking containers directory: {e}")
        
        # Return all paths with existence status
        results = []
        for path in potential_paths:
            results.append((path, path.exists()))
        
        return results

    # Check for command line arguments
    if len(sys.argv) > 1 and sys.argv[1] == "--debug-paths":
        print("Checking for potential NotePlan plugin paths...")
        paths = find_noteplan_plugins_paths()
        print("\nFound plugin paths:")
        for path, exists in paths:
            status = "EXISTS" if exists else "NOT FOUND"
            print(f"{status}: {path}")
        
        # Check which path the store would use
        store = PluginStore.get_instance()
        path = store._locate_noteplan_plugins_path()
        if path:
            print(f"\nThe application will use this path: {path}")
        else:
            print("\nNo valid plugin path found!")
        
        sys.exit(0)
        
    # Create and show the main window
    window = MainWindow()
    window.show()
    
    # Register cleanup function to ensure proper shutdown
    def cleanup():
        # Force cleanup of any remaining event filters or references
        app = QApplication.instance()
        if app:
            # Process any pending events
            app.processEvents()
            
            # Clean up any registered event filters
            if hasattr(window, 'banner_view'):
                app.removeEventFilter(window.banner_view)
    
    # Register cleanup function
    atexit.register(cleanup)
    
    # Run the application
    sys.exit(app.exec())