import json
import os
import datetime
import platform
from pathlib import Path
from dataclasses import dataclass
from typing import List, Dict, Optional, Union
from PyQt6.QtCore import QObject, pyqtSignal
from PyQt6.QtWidgets import QApplication


@dataclass
class PluginCommand:
    name: str
    description: str
    js_function: str
    
    @classmethod
    def from_dict(cls, data):
        return cls(
            name=data.get('name', ''),
            description=data.get('description', ''),
            js_function=data.get('jsFunction', '')
        )


@dataclass
class Plugin:
    id: str
    name: str
    version: str
    author: str
    description: str
    script: str
    commands: List[PluginCommand]
    
    @classmethod
    def from_dict(cls, data):
        return cls(
            id=data.get('plugin.id', ''),
            name=data.get('plugin.name', ''),
            version=data.get('plugin.version', ''),
            author=data.get('plugin.author', ''),
            description=data.get('plugin.description', ''),
            script=data.get('plugin.script', ''),
            commands=[PluginCommand.from_dict(cmd) for cmd in data.get('plugin.commands', [])]
        )


@dataclass
class RecentCommand:
    plugin_id: str
    plugin_name: str
    command_name: str
    command_function: str
    timestamp: datetime.datetime
    
    def to_dict(self):
        return {
            'plugin_id': self.plugin_id,
            'plugin_name': self.plugin_name,
            'command_name': self.command_name,
            'command_function': self.command_function,
            'timestamp': self.timestamp.isoformat()
        }
    
    @classmethod
    def from_dict(cls, data):
        return cls(
            plugin_id=data.get('plugin_id', ''),
            plugin_name=data.get('plugin_name', ''),
            command_name=data.get('command_name', ''),
            command_function=data.get('command_function', ''),
            timestamp=datetime.datetime.fromisoformat(data.get('timestamp', datetime.datetime.now().isoformat()))
        )


@dataclass
class FavoriteCommand:
    plugin_id: str
    plugin_name: str
    command_name: str
    command_function: str
    display_name: str = None  # Optional display name, if None use command_name
    
    def __post_init__(self):
        # Initialize display_name if not provided
        if self.display_name is None:
            self.display_name = self.command_name
    
    def to_dict(self):
        return {
            'plugin_id': self.plugin_id,
            'plugin_name': self.plugin_name,
            'command_name': self.command_name,
            'command_function': self.command_function,
            'display_name': self.display_name
        }
    
    @classmethod
    def from_dict(cls, data):
        return cls(
            plugin_id=data.get('plugin_id', ''),
            plugin_name=data.get('plugin_name', ''),
            command_name=data.get('command_name', ''),
            command_function=data.get('command_function', ''),
            display_name=data.get('display_name', data.get('command_name', ''))
        )


class PluginStore(QObject):
    plugins_updated = pyqtSignal()
    favorites_updated = pyqtSignal()
    recents_updated = pyqtSignal()
    loading_changed = pyqtSignal(bool)
    error_occurred = pyqtSignal(str)
    
    _instance = None
    
    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = PluginStore()
        return cls._instance
    
    def __init__(self):
        super().__init__()
        self.plugins: List[Plugin] = []
        self.recent_commands: List[RecentCommand] = []
        self.favorite_commands: List[FavoriteCommand] = []
        self.plugins_order: List[str] = []  # Store plugin IDs in order
        self.hidden_plugins: List[str] = []  # Store IDs of hidden plugins
        self.is_loading: bool = False
        self.error: Optional[str] = None
        
        # Load saved data
        self._load_recent_commands()
        self._load_favorite_commands()
        self._load_plugins_order()
        self._load_hidden_plugins()
    
    def scan_plugins(self):
        """Scan for NotePlan plugins"""
        self.is_loading = True
        self.loading_changed.emit(True)
        self.error = None
        
        plugins_path = self._locate_noteplan_plugins_path()
        if not plugins_path:
            self.error = "Could not find NotePlan plugins directory"
            self.error_occurred.emit(self.error)
            self.is_loading = False
            self.loading_changed.emit(False)
            return
        
        try:
            # Convert string path to Path object if needed
            plugins_path_obj = Path(plugins_path) if isinstance(plugins_path, str) else plugins_path
            
            self.plugins = []
            for folder_path in plugins_path_obj.iterdir():
                if folder_path.is_dir():
                    json_path = folder_path / "plugin.json"
                    
                    if json_path.is_file():
                        plugin = self._parse_plugin_json(str(json_path))
                        if plugin:
                            self.plugins.append(plugin)
            
            # Sort plugins by stored order if available, or by name as fallback
            if self.plugins_order:
                # Create a mapping of plugin IDs to index positions
                order_map = {plugin_id: idx for idx, plugin_id in enumerate(self.plugins_order)}
                
                # Sort plugins by their position in the order list
                # Plugins not in the order list will be sorted by name and placed at the end
                def plugin_sort_key(plugin):
                    if plugin.id in order_map:
                        return (0, order_map[plugin.id])  # Ordered plugins first, by their index
                    return (1, plugin.name)  # Unordered plugins last, alphabetically
                
                self.plugins.sort(key=plugin_sort_key)
            else:
                # If no order is stored, use alphabetical order
                self.plugins.sort(key=lambda p: p.name)
                
                # Initialize the order with current plugin IDs
                self.plugins_order = [plugin.id for plugin in self.plugins]
                self._save_plugins_order()
            
            self.plugins_updated.emit()
            
        except Exception as e:
            self.error = f"Error scanning plugins: {str(e)}"
            self.error_occurred.emit(self.error)
            import traceback
            traceback.print_exc()
        
        self.is_loading = False
        self.loading_changed.emit(False)
        
    def update_plugins_order(self, plugin_ids):
        """Update the order of plugins and save it to disk"""
        try:
            self.plugins_order = plugin_ids
            self._save_plugins_order()
        except Exception as e:
            print(f"Error updating plugins order: {e}")
    
    def _locate_noteplan_plugins_path(self):
        """Find the NotePlan plugins directory
        
        Checks for both Setapp and non-Setapp versions in various locations:
        1. Sandboxed Setapp: ~/Library/Containers/co.noteplan.NotePlan-setapp/Data/Library/Application Support/co.noteplan.NotePlan-setapp/Plugins
        2. Sandboxed regular: ~/Library/Containers/co.noteplan.NotePlan/Data/Library/Application Support/co.noteplan.NotePlan/Plugins
        3. Non-Setapp direct: ~/Library/Application Support/co.noteplan.NotePlan/Plugins
        4. Older NotePlan versions: ~/Library/Application Support/NotePlan/Plugins
        """
        from pathlib import Path
        home = Path.home()
        
        # Define potential paths where plugins could be located
        potential_paths = [
            # Sandboxed Setapp version
            home / "Library/Containers/co.noteplan.NotePlan-setapp/Data/Library/Application Support/co.noteplan.NotePlan-setapp/Plugins",
            
            # Normal sandboxed version
            home / "Library/Containers/co.noteplan.NotePlan/Data/Library/Application Support/co.noteplan.NotePlan/Plugins",
            
            # Direct non-sandboxed version 
            home / "Library/Application Support/co.noteplan.NotePlan/Plugins",
            
            # Older version
            home / "Library/Application Support/NotePlan/Plugins",
        ]
        
        # Check containers directory for any other NotePlan variants
        containers_path = home / "Library" / "Containers"
        if containers_path.exists():
            for container in containers_path.iterdir():
                if container.name.startswith("co.noteplan.NotePlan") and container.is_dir():
                    container_path = container / "Data" / "Library" / "Application Support" / container.name / "Plugins"
                    # Only add if not already in our list
                    if container_path not in potential_paths:
                        potential_paths.append(container_path)
        
        # Return the first path that exists
        for path in potential_paths:
            if path.exists():
                print(f"Found NotePlan plugins at: {path}")
                return str(path)
        
        print("Could not find any NotePlan plugins directory")
        return None
    
    def _parse_plugin_json(self, json_path):
        """Parse plugin.json file"""
        try:
            # Convert to Path object if it's a string
            path_obj = Path(json_path) if isinstance(json_path, str) else json_path
            
            with path_obj.open('r', encoding='utf-8') as f:
                data = json.load(f)
                return Plugin.from_dict(data)
        except Exception as e:
            print(f"Error parsing plugin JSON at {json_path}: {e}")
            return None
    
    def execute_command(self, plugin: Plugin, command: PluginCommand, command_key=None):
        """Execute a NotePlan plugin command
        
        Args:
            plugin: The Plugin object
            command: The PluginCommand object
            command_key: Optional explicit command name to use (for favorites with custom display names)
        """
        try:
            # Explicitly import these modules right here
            import subprocess
            import time
            import webbrowser
            
            # Try multiple methods to ensure the command gets through
            plugin_id = plugin.id.replace(" ", "%20")
            
            # If a custom command key is provided, use that instead of the command name
            # This is needed for favorites which have custom display names
            if command_key:
                command_name = command_key.replace(" ", "%20")
            else:
                # Otherwise use the command name from the command object
                command_name = command.name.replace(" ", "%20")
            
            # Construct the URL with proper encoding
            url = f"noteplan://x-callback-url/runPlugin?pluginID={plugin_id}&command={command_name}"
            print(f"Executing command: {url}")
            
            # Platform-specific command execution
            success = False
            
            # First try to directly open NotePlan
            if platform.system() == "Darwin":  # macOS
                try:
                    # On macOS, use the open command to launch NotePlan
                    subprocess.run(["open", "-a", "NotePlan"], check=False)
                    time.sleep(0.3)  # Wait for NotePlan to initialize
                except Exception as e:
                    print(f"Failed to launch NotePlan: {e}")
            elif platform.system() == "Windows":
                # On Windows, we cannot directly launch NotePlan, but will try URL methods
                print("Windows detected, will try URL methods directly")
                time.sleep(0.2)
            else:
                # On Linux systems, we'll just try the URL methods
                print("Non-macOS/Windows platform detected, will try URL methods directly")
                time.sleep(0.2)
            
            # Method 1: Try platform-specific direct URL handling (most reliable)
            try:
                if platform.system() == "Darwin":  # macOS
                    # On macOS, use the open command
                    subprocess.run(["open", url], check=False)
                else:
                    # On other platforms, use webbrowser module
                    webbrowser.open(url)
                success = True
                print("Method 1 succeeded")
            except Exception as e:
                print(f"Method 1 failed: {e}")
                
            # Only try method 2 if method 1 failed
            if not success:
                # Wait briefly between methods
                time.sleep(0.2)
                
                # Method 2: Use platform-specific fallbacks
                try:
                    if platform.system() == "Darwin":  # macOS
                        # Use AppleScript as fallback on macOS
                        applescript = f'tell application "NotePlan" to activate\ntell application "System Events" to open location "{url}"'
                        subprocess.run(["osascript", "-e", applescript], check=False)
                    else:
                        # On other platforms, try another webbrowser approach
                        webbrowser.get().open(url)
                    success = True
                    print("Method 2 succeeded")
                except Exception as e:
                    print(f"Method 2 failed: {e}")
            
            # Only try method 3 if both previous methods failed
            if not success:
                # Method 3: Last resort options
                try:
                    if platform.system() == "Darwin":  # macOS
                        # Another AppleScript variant for macOS
                        applescript2 = f'do shell script "open \\"{url}\\""'
                        subprocess.run(["osascript", "-e", applescript2], check=False)
                    elif platform.system() == "Windows":
                        # On Windows, try using the start command
                        subprocess.run(["start", url], shell=True, check=False)
                    else:
                        # On Linux, try xdg-open
                        subprocess.run(["xdg-open", url], check=False)
                    print("Method 3 succeeded")
                except Exception as e:
                    print(f"Method 3 failed: {e}")
            
            # Add to recent commands
            recent = RecentCommand(
                plugin_id=plugin.id,
                plugin_name=plugin.name,
                command_name=command.name,
                command_function=command.js_function,
                timestamp=datetime.datetime.now()
            )
            
            # Remove if already exists
            self.recent_commands = [r for r in self.recent_commands 
                                if not (r.plugin_id == plugin.id and 
                                        r.command_function == command.js_function)]
            
            # Add to beginning of list
            self.recent_commands.insert(0, recent)
            
            # Keep only most recent 20
            self.recent_commands = self.recent_commands[:20]
            
            # Save and notify
            self._save_recent_commands()
            self.recents_updated.emit()
        except Exception as e:
            print(f"Error executing command: {e}")
            import traceback
            traceback.print_exc()
    
    def toggle_favorite(self, plugin: Plugin, command: PluginCommand):
        """Toggle a command as favorite"""
        try:
            # Check if already favorited
            is_favorite = False
            for i, fav in enumerate(self.favorite_commands):
                if fav.plugin_id == plugin.id and fav.command_function == command.js_function:
                    # Remove if found
                    self.favorite_commands.pop(i)
                    is_favorite = True
                    break
            
            # Add if not already a favorite
            if not is_favorite:
                favorite = FavoriteCommand(
                    plugin_id=plugin.id,
                    plugin_name=plugin.name,
                    command_name=command.name,
                    command_function=command.js_function,
                    display_name=command.name  # Initialize display_name with original command name
                )
                self.favorite_commands.append(favorite)
            
            # Save favorites immediately
            self._save_favorite_commands()
            
            # Process any UI events first to avoid recursion issues
            app = QApplication.instance()
            if app:
                app.processEvents()
            
            # Emit signal to update all views that show favorites
            try:
                self.favorites_updated.emit()
            except Exception as e:
                print(f"Error emitting favorites_updated: {e}")
                
            # Let's NOT emit plugins_updated as it's causing crashes
            # The favorites_updated signal should be sufficient
        except Exception as e:
            print(f"Error toggling favorite: {e}")
    
    def is_favorite(self, plugin: Plugin, command: PluginCommand) -> bool:
        """Check if a command is favorited"""
        return any(f.plugin_id == plugin.id and f.command_function == command.js_function 
                  for f in self.favorite_commands)
                  
    def reorder_favorites(self, source_idx: int, target_idx: int):
        """Reorder favorites by moving item from source_idx to target_idx"""
        try:
            # Validate indices
            if source_idx < 0 or source_idx >= len(self.favorite_commands):
                return
            
            # Get the item to move
            item = self.favorite_commands.pop(source_idx)
            
            # If the target is now beyond the end of the list, adjust it
            if target_idx > len(self.favorite_commands):
                target_idx = len(self.favorite_commands)
            
            # Insert at the new position
            self.favorite_commands.insert(target_idx, item)
            
            # Save changes immediately
            self._save_favorite_commands()
            
            # Process any UI events first to avoid recursion issues
            app = QApplication.instance()
            if app:
                app.processEvents()
            
            # Notify listeners
            try:
                self.favorites_updated.emit()
            except Exception as e:
                print(f"Error emitting favorites_updated: {e}")
        except Exception as e:
            print(f"Error reordering favorites: {e}")
            
    def update_favorite_display_name(self, index: int, new_name: str):
        """Update the display name of a favorite command at the specified index"""
        try:
            # Validate index
            if index < 0 or index >= len(self.favorite_commands):
                print(f"Invalid favorite index: {index}")
                return
                
            # Get old name for comparison
            old_display_name = self.favorite_commands[index].display_name
            print(f"PluginStore: Updating favorite at index {index} from '{old_display_name}' to '{new_name}'")
                
            # Update the display name (keeping the original command_name unchanged)
            self.favorite_commands[index].display_name = new_name
            
            # Save changes immediately
            self._save_favorite_commands()
            print(f"PluginStore: Saved favorite commands to disk with updated display name")
            
            # Skip emitting the signal to avoid triggering UI rebuild
            # This is likely causing the blank screen issue
            print(f"PluginStore: Skipping favorites_updated signal to avoid UI rebuild")
            
        except Exception as e:
            print(f"Error updating favorite display name: {e}")
    
    def _get_data_file_path(self, filename):
        """Get path to store app data based on platform
        
        On macOS: ~/Library/Application Support/NotePlanLauncher/
        On Windows: ~/AppData/Roaming/NotePlanLauncher/
        On Linux: ~/.config/NotePlanLauncher/
        """
        home = Path.home()
        app_name = "NotePlanLauncher"
        
        # Platform-specific app data locations
        if platform.system() == "Darwin":  # macOS
            app_data_dir = home / "Library" / "Application Support" / app_name
        elif platform.system() == "Windows":
            app_data_dir = home / "AppData" / "Roaming" / app_name
        else:  # Linux/Unix
            app_data_dir = home / ".config" / app_name
        
        # Create directory if it doesn't exist
        app_data_dir.mkdir(parents=True, exist_ok=True)
        
        # Return full path to the file
        return str(app_data_dir / filename)
    
    def _load_recent_commands(self):
        """Load recent commands from disk"""
        try:
            path = Path(self._get_data_file_path("recent_commands.json"))
            if path.exists():
                with path.open('r') as f:
                    data = json.load(f)
                    self.recent_commands = [RecentCommand.from_dict(item) for item in data]
        except Exception as e:
            print(f"Error loading recent commands: {e}")
            self.recent_commands = []
    
    def _save_recent_commands(self):
        """Save recent commands to disk"""
        try:
            path = Path(self._get_data_file_path("recent_commands.json"))
            with path.open('w') as f:
                data = [cmd.to_dict() for cmd in self.recent_commands]
                json.dump(data, f, indent=2)
        except Exception as e:
            print(f"Error saving recent commands: {e}")
    
    def _load_favorite_commands(self):
        """Load favorite commands from disk"""
        try:
            path = Path(self._get_data_file_path("favorite_commands.json"))
            if path.exists():
                with path.open('r') as f:
                    data = json.load(f)
                    self.favorite_commands = [FavoriteCommand.from_dict(item) for item in data]
        except Exception as e:
            print(f"Error loading favorite commands: {e}")
            self.favorite_commands = []
    
    def _save_favorite_commands(self):
        """Save favorite commands to disk"""
        try:
            path = Path(self._get_data_file_path("favorite_commands.json"))
            with path.open('w') as f:
                data = [cmd.to_dict() for cmd in self.favorite_commands]
                json.dump(data, f, indent=2)
        except Exception as e:
            print(f"Error saving favorite commands: {e}")
            
    def _load_plugins_order(self):
        """Load plugins order from disk"""
        try:
            path = Path(self._get_data_file_path("plugins_order.json"))
            if path.exists():
                with path.open('r') as f:
                    self.plugins_order = json.load(f)
        except Exception as e:
            print(f"Error loading plugins order: {e}")
            self.plugins_order = []
    
    def _save_plugins_order(self):
        """Save plugins order to disk"""
        try:
            path = Path(self._get_data_file_path("plugins_order.json"))
            with path.open('w') as f:
                json.dump(self.plugins_order, f, indent=2)
        except Exception as e:
            print(f"Error saving plugins order: {e}")

    def _load_hidden_plugins(self):
        """Load hidden plugins from disk"""
        try:
            path = Path(self._get_data_file_path("hidden_plugins.json"))
            if path.exists():
                with path.open('r') as f:
                    self.hidden_plugins = json.load(f)
        except Exception as e:
            print(f"Error loading hidden plugins: {e}")
            self.hidden_plugins = []
    
    def _save_hidden_plugins(self):
        """Save hidden plugins to disk"""
        try:
            path = Path(self._get_data_file_path("hidden_plugins.json"))
            with path.open('w') as f:
                json.dump(self.hidden_plugins, f, indent=2)
        except Exception as e:
            print(f"Error saving hidden plugins: {e}")
            
    def toggle_plugin_visibility(self, plugin_id):
        """Toggle a plugin's visibility in the banner"""
        if plugin_id in self.hidden_plugins:
            self.hidden_plugins.remove(plugin_id)
        else:
            self.hidden_plugins.append(plugin_id)
        
        # Save changes
        self._save_hidden_plugins()
        
        # Emit signal to update UI
        self.plugins_updated.emit()
        
    def is_plugin_hidden(self, plugin_id):
        """Check if a plugin is hidden"""
        return plugin_id in self.hidden_plugins