import subprocess
import time

# Try to import sip, but provide a fallback if not available
try:
    import sip
    HAS_SIP = True
except ImportError:
    HAS_SIP = False
    # Define a simple function to use when sip is not available
    def isdeleted(obj):
        # Simple check based on the assumption that a deleted object will raise AttributeError
        try:
            # Try to access a common attribute or method
            getattr(obj, "isVisible", None)
            return False
        except (RuntimeError, AttributeError):
            # Either the C++ object is gone or the attribute doesn't exist
            return True
else:
    # If sip is available, use its isdeleted function
    isdeleted = sip.isdeleted

from PyQt6.QtCore import Qt, QSize, QPropertyAnimation, QEasingCurve, QTimer, pyqtSignal, QMimeData
from PyQt6.QtGui import QIcon, QColor, QPalette, QFont, QPainter, QPainterPath, QBrush, QDrag, QAction
from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLabel, 
    QPushButton, QScrollArea, QFrame, QGraphicsDropShadowEffect,
    QSizePolicy, QApplication, QToolButton, QGridLayout, QLineEdit,
    QStyle, QMenu
)

from models import PluginStore, Plugin, PluginCommand


class RoundedWidget(QWidget):
    """A widget with rounded corners and shadow"""
    
    def __init__(self, parent=None, bg_color="#ffffff", radius=15):
        super().__init__(parent)
        self.bg_color = bg_color
        self.radius = radius
        self.setAutoFillBackground(False)
        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground, False)
        
        # Add shadow
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(20)
        shadow.setColor(QColor(0, 0, 0, 50))
        shadow.setOffset(0, 4)
        self.setGraphicsEffect(shadow)
    
    def paintEvent(self, event):
        """Custom paint event for rounded corners"""
        painter = QPainter(self)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        
        # Create the rounded rect path
        path = QPainterPath()
        path.addRoundedRect(0, 0, self.width(), self.height(), self.radius, self.radius)
        
        # Fill with background color
        painter.fillPath(path, QBrush(QColor(self.bg_color)))


class CommandItem(QWidget):
    """Widget representing a single command in the popup panel"""
    
    def __init__(self, plugin, command, parent=None):
        super().__init__(parent)
        self.plugin = plugin
        self.command = command
        self.store = PluginStore.get_instance()
        self.popup_parent = parent
        
        # Create layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(10, 8, 10, 8)
        layout.setSpacing(10)
        
        # Command info with enhancement for custom display names
        self.name_label = QLabel(f"<b>{command.name}</b>")
        self.name_label.setStyleSheet("font-size: 13px;")
        self.name_label.setWordWrap(True)  # Enable word wrap for long command names
        self.name_label.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        
        # Set tooltip with both command name and description
        tooltip_text = command.name
        if hasattr(self, 'custom_name') and self.custom_name:
            # If this is a custom name, include original name in tooltip
            tooltip_text += f" (custom name)"
        if command.description:
            tooltip_text += f"\n\n{command.description}"
        self.name_label.setToolTip(tooltip_text)
        
        layout.addWidget(self.name_label, 1)
        
        # Favorite button - using heart emoji
        self.fav_button = QToolButton()
        self.fav_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.fav_button.setToolTip("Add to favorites")
        self.fav_button.setStyleSheet("""
            QToolButton {
                font-size: 16px;
                background: transparent;
                border: none;
            }
        """)
        # Use explicit lambda
        self.fav_button.clicked.connect(lambda checked=False: self.toggle_favorite())
        layout.addWidget(self.fav_button)
        
        # Run button with improved visibility
        run_button = QPushButton("Run")
        run_button.setCursor(Qt.CursorShape.PointingHandCursor)
        run_button.setMinimumWidth(60)
        run_button.setFixedHeight(30)
        run_button.setStyleSheet("""
            QPushButton {
                background-color: #4a90e2;
                color: white;
                border: none;
                padding: 5px 10px;
                border-radius: 4px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #3a80d2;
            }
            QPushButton:pressed {
                background-color: #2a70c2;
            }
        """)
        # Use explicit lambda
        run_button.clicked.connect(lambda checked=False: self.run_command())
        layout.addWidget(run_button)
        
        # Set a fixed height to ensure consistent sizing
        self.setFixedHeight(44)
        
        # Update favorite status
        self.update_favorite_status()
        self.store.favorites_updated.connect(self.update_favorite_status)
    
    def toggle_favorite(self):
        """Toggle favorite status for this command"""
        try:
            # Ensure necessary imports are available
            import subprocess
            import time
            
            # Prevent event from closing popup
            QApplication.instance().setOverrideCursor(Qt.CursorShape.WaitCursor)
            
            # Check if we have all the necessary components before executing
            if not hasattr(self, 'plugin') or not hasattr(self, 'command'):
                print("Error: Missing plugin or command data")
                QApplication.instance().restoreOverrideCursor()
                return
                
            # Toggle the favorite
            self.store.toggle_favorite(self.plugin, self.command)
            self.update_favorite_status()
            
            # Force the banner view to update
            if hasattr(self, 'popup_parent') and hasattr(self.popup_parent, 'parent_widget'):
                # First close any popups
                self.popup_parent.parent_widget.close_active_popup()
                
                # Then update the banner view after a brief delay
                QTimer.singleShot(100, self.popup_parent.parent_widget.update_plugins)
            
            # Restore cursor
            QApplication.instance().restoreOverrideCursor()
        except Exception as e:
            # Make sure cursor is restored
            try:
                QApplication.instance().restoreOverrideCursor()
            except:
                pass
            print(f"Error toggling favorite: {e}")
        
    def update_favorite_status(self):
        """Update favorite button appearance - using heart emoji"""
        is_favorite = self.store.is_favorite(self.plugin, self.command)
        if is_favorite:
            self.fav_button.setText("❤️")
            self.fav_button.setStyleSheet("""
                QToolButton {
                    font-size: 16px;
                    background: transparent;
                    border: none;
                }
            """)
            self.fav_button.setToolTip("Remove from favorites")
        else:
            self.fav_button.setText("🤍")
            self.fav_button.setStyleSheet("""
                QToolButton {
                    font-size: 16px;
                    background: transparent;
                    border: none;
                }
            """)
            self.fav_button.setToolTip("Add to favorites")
            
    def run_command(self):
        """Run the command and close popup"""
        try:
            # Direct imports to ensure they're available
            import subprocess
            import time
            
            # Prevent event from closing popup prematurely
            QApplication.instance().setOverrideCursor(Qt.CursorShape.WaitCursor)
            
            # Check if we have all the necessary components before executing
            if not hasattr(self, 'plugin') or not hasattr(self, 'command'):
                print("Error: Missing plugin or command data")
                QApplication.instance().restoreOverrideCursor()
                return
            
            # For favorites with custom display names, we need to find the original command name
            original_command_name = None
            if hasattr(self, 'custom_name') and self.custom_name:
                # Find the original command to get its name
                for cmd in self.plugin.commands:
                    if cmd.js_function == self.command.js_function:
                        original_command_name = cmd.name
                        print(f"Using original command name: {original_command_name} (was: {self.command.name})")
                        break
            
            # Execute through the store - this will handle all execution methods
            try:
                # Pass the original command name if this is a custom named favorite
                self.store.execute_command(
                    self.plugin, 
                    self.command,
                    command_key=original_command_name  # Will be None for regular commands
                )
            except Exception as store_error:
                print(f"Error executing command: {store_error}")
                
                # Only log the error, no fallback execution since that would duplicate the command
                print(f"Command execution failed for: {self.command.name}")
            
            # Only close popup after command is executed
            if hasattr(self, 'popup_parent') and hasattr(self.popup_parent, 'parent_widget') and hasattr(self.popup_parent.parent_widget, 'close_active_popup'):
                try:
                    # Use a timer to delay closing the popup
                    QTimer.singleShot(500, self.popup_parent.parent_widget.close_active_popup)
                except Exception as popup_e:
                    print(f"Error closing popup: {popup_e}")
            
            # Restore cursor
            QApplication.instance().restoreOverrideCursor()
        except Exception as e:
            try:
                QApplication.instance().restoreOverrideCursor()
            except:
                pass
            print(f"Error executing command: {e}")
    
    def contextMenuEvent(self, event):
        """Handle right-click properly"""
        # Just prevent the default behavior which might be causing crashes
        # We could implement a custom menu here in the future
        event.accept()


class PluginPopup(RoundedWidget):
    """Popup panel displaying commands for a plugin"""
    
    def __init__(self, plugin, parent=None):
        bg_color = "#ffffff" if not parent.dark_mode else "#2d2d2d"
        super().__init__(parent, bg_color=bg_color)
        self.plugin = plugin
        self.parent_widget = parent
        
        # Set popup to be frameless but NOT a popup window type initially
        # This ensures it's correctly parented and doesn't create input issues
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.NoDropShadowWindowHint)
        
        # Ensure it doesn't steal focus
        self.setAttribute(Qt.WidgetAttribute.WA_ShowWithoutActivating)
        
        # Create layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(5, 5, 5, 5)
        layout.setSpacing(0)
        
        # Plugin title
        title_widget = QWidget()
        title_layout = QHBoxLayout(title_widget)
        title_layout.setContentsMargins(10, 5, 10, 5)
        
        name_label = QLabel(plugin.name)
        name_label.setStyleSheet("font-size: 14px; font-weight: bold;")
        
        title_layout.addWidget(name_label)
        title_layout.addStretch(1)
        
        # Close button
        close_button = QToolButton()
        close_button.setText("×")
        close_button.setToolTip("Close")
        close_button.setCursor(Qt.CursorShape.PointingHandCursor)
        close_button.setStyleSheet("""
            QToolButton {
                font-size: 20px;
                background: transparent;
                border: none;
            }
        """)
        close_button.clicked.connect(self.close_popup)
        title_layout.addWidget(close_button)
        
        layout.addWidget(title_widget)
        
        # Add separator
        separator = QFrame()
        separator.setFrameShape(QFrame.Shape.HLine)
        separator.setFrameShadow(QFrame.Shadow.Sunken)
        separator.setStyleSheet("background-color: #dddddd;" if not parent.dark_mode else "background-color: #444444;")
        layout.addWidget(separator)
        
        # Scroll area for commands - with auto-scrollbar behavior
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        
        # Container for commands
        commands_widget = QWidget()
        self.commands_layout = QVBoxLayout(commands_widget)
        self.commands_layout.setContentsMargins(5, 5, 5, 5)
        self.commands_layout.setSpacing(2)
        self.commands_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        
        # Add command items
        for command in plugin.commands:
            command_item = CommandItem(plugin, command)
            self.commands_layout.addWidget(command_item)
        
        # Empty state if no commands
        if not plugin.commands:
            empty_label = QLabel("No commands available")
            empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            empty_label.setStyleSheet("color: #888888; padding: 20px;")
            self.commands_layout.addWidget(empty_label)
        
        scroll_area.setWidget(commands_widget)
        layout.addWidget(scroll_area)
        
        # Set minimum width but allow expansion for long command names
        self.setMinimumWidth(400)
        self.setMaximumWidth(600)  # Allow wider popups for long command names
        
        # We'll let the height be determined by content, with reasonable limits
        self.setMinimumHeight(100)
        self.setMaximumHeight(600)  # Larger max height to prevent scrolling in most cases
        
        # Hide initially
        self.setVisible(False)
    
    def close_popup(self):
        """Close the popup"""
        self.setVisible(False)
        self.parent_widget.active_popup = None


class PluginButton(QToolButton):
    """Button for a plugin in the banner"""
    
    def __init__(self, plugin, parent=None):
        super().__init__(parent)
        self.plugin = plugin
        self.parent_widget = parent
        self.drag_start_position = None
        self.click_detected = False  # Flag to distinguish clicks from drags
        
        # Extract first character as emoji or use fallback
        if plugin.name and len(plugin.name) > 0:
            first_char = plugin.name[0]
        else:
            first_char = "📦"  # Default emoji if no name
        
        # Set emoji as text
        self.setText(first_char)
        
        # Set tooltip with plugin name
        self.setToolTip(plugin.name)
        self.setToolTipDuration(5000)  # Show tooltip for 5 seconds
        
        # Set size and styles
        self.setMinimumSize(70, 70)
        self.setMaximumSize(70, 70)
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        
        # Set larger font for emoji
        self.setStyleSheet("""
            QToolButton {
                font-size: 36px;
                background: transparent;
                border: none;
                color: #333333;
                padding: 5px;
            }
            QToolButton:hover {
                background-color: rgba(0, 0, 0, 0.05);
                border-radius: 10px;
            }
            QToolButton:pressed {
                background-color: rgba(0, 0, 0, 0.1);
                border-radius: 10px;
            }
        """)
        
        # Setup mouse tracking and enable drag and drop
        self.setMouseTracking(True)
        self.setAcceptDrops(True)
    
    def mouseReleaseEvent(self, event):
        """Handle mouse release to show popup menu on click"""
        if event.button() == Qt.MouseButton.LeftButton:
            # Only show popup if this was a click, not the end of a drag
            if not self.drag_start_position or \
               (event.position() - self.drag_start_position).manhattanLength() < QApplication.startDragDistance():
                
                # Track previous popup status
                previous_popup = self.parent_widget.active_popup
                previous_owner = self.parent_widget.active_popup_owner
                
                # Always close any existing popup first
                if self.parent_widget.active_popup is not None:
                    self.parent_widget.close_active_popup()
                
                # Wait briefly to ensure UI updates
                QApplication.processEvents()
                
                # Open new popup if this isn't the button that was just closed
                # or if no popup was previously open
                if previous_popup is None or previous_owner != self:
                    # Show the plugin popup with a slight delay to ensure proper UI state
                    QTimer.singleShot(50, lambda: self.parent_widget.show_plugin_popup(self.plugin, False))
                
        self.drag_start_position = None
        super().mouseReleaseEvent(event)
    
    def mousePressEvent(self, event):
        """Handle mouse press for drag and drop"""
        if event.button() == Qt.MouseButton.LeftButton:
            self.drag_start_position = event.position()
        super().mousePressEvent(event)
    
    def mouseMoveEvent(self, event):
        """Handle mouse move for drag and drop"""
        # Check for valid drag initialization
        if not (event.buttons() & Qt.MouseButton.LeftButton):
            return
        if not self.drag_start_position:
            return
        
        # Check if distance is far enough to start a drag
        distance = (event.position() - self.drag_start_position).manhattanLength()
        if distance < QApplication.startDragDistance():
            return
        
        # Start drag operation
        drag = QDrag(self)
        mimedata = QMimeData()
        
        # Get index in layout to use in drop handling
        for i in range(self.parent_widget.banner_layout.count()):
            if self.parent_widget.banner_layout.itemAt(i).widget() == self:
                mimedata.setData("application/x-button-index", str(i).encode())
                break
        
        drag.setMimeData(mimedata)
        
        # Set drag pixmap (an image of the button during drag)
        pixmap = self.grab()
        drag.setPixmap(pixmap)
        drag.setHotSpot(event.position().toPoint())
        
        # Execute drag
        drag.exec(Qt.DropAction.MoveAction)
    
    def update_style(self, dark_mode):
        """Update style based on dark mode"""
        if dark_mode:
            self.setStyleSheet("""
                QToolButton {
                    font-size: 36px;
                    background: transparent;
                    border: none;
                    color: #f0f0f0;
                    padding: 5px;
                }
                QToolButton:hover {
                    background-color: rgba(255, 255, 255, 0.1);
                    border-radius: 10px;
                }
                QToolButton:pressed {
                    background-color: rgba(255, 255, 255, 0.15);
                    border-radius: 10px;
                }
            """)
        else:
            self.setStyleSheet("""
                QToolButton {
                    font-size: 36px;
                    background: transparent;
                    border: none;
                    color: #333333;
                    padding: 5px;
                }
                QToolButton:hover {
                    background-color: rgba(0, 0, 0, 0.05);
                    border-radius: 10px;
                }
                QToolButton:pressed {
                    background-color: rgba(0, 0, 0, 0.1);
                    border-radius: 10px;
                }
            """)


class BannerView(QWidget):
    """Main banner view with plugin emoji buttons and popups"""
    
    # Signal for theme toggle
    theme_toggle_requested = pyqtSignal()
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.store = PluginStore.get_instance()
        self.main_window = parent
        self.dark_mode = False if parent is None else parent.dark_mode
        self.active_popup = None
        self.active_popup_owner = None
        self.plugin_buttons = {}
        self.plugin_popups = {}
        
        # Main layout - strictly vertical with no spacing
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.setSpacing(0)
        
        # Banner with plugin emojis - takes up entire window height
        self.banner = QWidget()
        self.banner.setFixedHeight(80)
        self.banner.setStyleSheet("background-color: #FDF6E3;")  # Set light cream background
        
        # Use horizontal layout for emojis
        self.banner_layout = QHBoxLayout(self.banner)
        self.banner_layout.setContentsMargins(10, 0, 10, 0)  # Reduce vertical padding
        self.banner_layout.setSpacing(5)  # Make emojis closer together
        self.banner_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        main_layout.addWidget(self.banner)
        
        # Content area for popups (hidden but used to hold popup widgets)
        self.content_area = QWidget()
        self.content_layout = QVBoxLayout(self.content_area)
        self.content_layout.setContentsMargins(0, 0, 0, 0)
        self.content_layout.setSpacing(0)
        main_layout.addWidget(self.content_area)
        self.content_area.setVisible(False)  # Hidden, popups will float on their own
        
        # Setup drag and drop for reordering
        self.setAcceptDrops(True)
        self.banner.setAcceptDrops(True)
        self.banner.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.banner.customContextMenuRequested.connect(self.on_context_menu_requested)
        
        # Create theme toggle button that will be added last
        self.theme_button = QToolButton()
        self.theme_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.theme_button.setMinimumSize(60, 70)
        self.theme_button.setMaximumSize(60, 70)
        self.theme_button.setStyleSheet("""
            QToolButton {
                font-size: 24px;
                background: transparent;
                border: none;
                padding: 5px;
            }
            QToolButton:hover {
                background-color: rgba(0, 0, 0, 0.05);
                border-radius: 10px;
            }
            QToolButton:pressed {
                background-color: rgba(0, 0, 0, 0.1);
                border-radius: 10px;
            }
        """)
        self.theme_button.clicked.connect(self.request_theme_toggle)
        self.update_theme_button()
        
        # Install event filter on the application to handle clicks outside popups
        QApplication.instance().installEventFilter(self)
        
        # Connect signals
        self.store.plugins_updated.connect(self.update_plugins)
        self.store.favorites_updated.connect(self.on_favorites_updated)
        
        # Initialize with current plugins
        self.update_plugins()
    
    def request_theme_toggle(self):
        """Emit signal to request theme toggle from main window"""
        self.theme_toggle_requested.emit()
        
    def on_context_menu_requested(self, position):
        """Handle right-click on banner to toggle theme"""
        # Just toggle the theme without showing a context menu
        self.request_theme_toggle()
    
    def update_theme_button(self):
        """Update theme button appearance"""
        try:
            if hasattr(self, 'theme_button') and self.theme_button is not None:
                if self.dark_mode:
                    self.theme_button.setText("☀️")
                    self.theme_button.setToolTip("Switch to Light Mode")
                    self.theme_button.setStyleSheet("""
                        QToolButton {
                            font-size: 24px;
                            background: transparent;
                            border: none;
                            color: #f0f0f0;
                            padding: 5px;
                        }
                        QToolButton:hover {
                            background-color: rgba(255, 255, 255, 0.1);
                            border-radius: 10px;
                        }
                        QToolButton:pressed {
                            background-color: rgba(255, 255, 255, 0.15);
                            border-radius: 10px;
                        }
                    """)
                else:
                    self.theme_button.setText("🌙")
                    self.theme_button.setToolTip("Switch to Dark Mode")
                    self.theme_button.setStyleSheet("""
                        QToolButton {
                            font-size: 24px;
                            background: transparent;
                            border: none;
                            color: #333333;
                            padding: 5px;
                        }
                        QToolButton:hover {
                            background-color: rgba(0, 0, 0, 0.05);
                            border-radius: 10px;
                        }
                        QToolButton:pressed {
                            background-color: rgba(0, 0, 0, 0.1);
                            border-radius: 10px;
                        }
                    """)
        except RuntimeError:
            # Skip if button has been deleted
            pass
    
    def dragEnterEvent(self, event):
        """Allow drag events for reordering buttons"""
        if event.mimeData().hasFormat("application/x-button-index"):
            event.accept()
        else:
            event.ignore()
    
    def dragMoveEvent(self, event):
        """Handle drag movement"""
        if event.mimeData().hasFormat("application/x-button-index"):
            event.accept()
        else:
            event.ignore()
    
    def dropEvent(self, event):
        """Handle drop to reorder buttons"""
        if event.mimeData().hasFormat("application/x-button-index"):
            source_idx = int(event.mimeData().data("application/x-button-index"))
            
            # Get the target position
            drop_pos = event.position().toPoint()
            target_idx = -1
            
            # Find the widget at the drop position
            for i in range(self.banner_layout.count()):
                widget = self.banner_layout.itemAt(i).widget()
                if widget:
                    widget_geo = widget.geometry()
                    
                    # If drop is on left half of widget, insert before; if right half, insert after
                    if widget_geo.contains(drop_pos):
                        if drop_pos.x() < widget_geo.x() + widget_geo.width() / 2:
                            target_idx = i
                        else:
                            target_idx = i + 1
                        break
            
            # If we didn't find a specific position, append to the end
            if target_idx == -1:
                target_idx = self.banner_layout.count()
            
            # Don't do anything if dropping in the same place
            if source_idx == target_idx or source_idx + 1 == target_idx:
                event.accept()
                return
            
            # Get the widget to move
            source_widget = self.banner_layout.takeAt(source_idx).widget()
            
            # Adjust target index if moving from earlier to later position
            if source_idx < target_idx:
                target_idx -= 1
            
            # Insert at the new position
            self.banner_layout.insertWidget(target_idx, source_widget)
            
            # Save the new plugin order
            self._save_plugin_order()
            
            event.accept()
        else:
            event.ignore()
            
    def _save_plugin_order(self):
        """Save the current order of plugins to the store"""
        try:
            # Collect plugin IDs in the current order
            plugin_ids = []
            
            # Skip the favorites button and theme button
            for i in range(self.banner_layout.count()):
                widget = self.banner_layout.itemAt(i).widget()
                
                # Check if it's a plugin button (has the plugin attribute)
                if widget and hasattr(widget, 'plugin') and hasattr(widget.plugin, 'id'):
                    plugin_ids.append(widget.plugin.id)
            
            # Update the order in the store
            if plugin_ids:
                self.store.update_plugins_order(plugin_ids)
        except Exception as e:
            print(f"Error saving plugin order: {e}")
    
    def update_plugins(self):
        """Update the banner with plugin buttons"""
        # Store the theme button before clearing layout
        theme_button = None
        if hasattr(self, 'theme_button') and self.theme_button is not None:
            try:
                # Remove theme button from layout without deleting it
                self.banner_layout.removeWidget(self.theme_button)
                theme_button = self.theme_button
            except:
                # Create a new theme button if needed
                pass
        
        # Clear existing buttons
        for i in reversed(range(self.banner_layout.count())):
            widget = self.banner_layout.itemAt(i).widget()
            if widget:
                self.banner_layout.removeWidget(widget)
                widget.deleteLater()
        
        # Clear existing popups
        for popup in self.plugin_popups.values():
            popup.deleteLater()
        
        self.plugin_buttons = {}
        self.plugin_popups = {}
        
        # Always add favorites button, regardless of whether there are favorites
        favorites_button = QToolButton()
        favorites_button.setText("❤️")  # Heart emoji instead of star
        favorites_button.setToolTip("Favorites")
        favorites_button.setMinimumSize(70, 70)
        favorites_button.setMaximumSize(70, 70)
        favorites_button.setCursor(Qt.CursorShape.PointingHandCursor)
        
        favorites_button.setStyleSheet("""
            QToolButton {
                font-size: 36px;
                background: transparent;
                border: none;
                padding: 5px;
            }
            QToolButton:hover {
                background-color: rgba(0, 0, 0, 0.05);
                border-radius: 10px;
            }
            QToolButton:pressed {
                background-color: rgba(0, 0, 0, 0.1);
                border-radius: 10px;
            }
        """)
        
        # Set up click behavior for favorites with a slight delay to ensure proper UI state
        favorites_button.clicked.connect(lambda: QTimer.singleShot(50, lambda: self.show_favorites_popup(favorites_button)))
        favorites_button.is_favorites = True
        
        self.banner_layout.addWidget(favorites_button)
        self.favorites_button = favorites_button
        
        # Create favorites popup
        self.create_favorites_popup()
        
        # Add plugin buttons (only for non-hidden plugins)
        for plugin in self.store.plugins:
            # Skip hidden plugins
            if self.store.is_plugin_hidden(plugin.id):
                continue
                
            button = PluginButton(plugin, self)
            self.banner_layout.addWidget(button)
            self.plugin_buttons[plugin.id] = button
            
            # Create popup for this plugin
            popup = PluginPopup(plugin, self)
            self.plugin_popups[plugin.id] = popup
            self.content_layout.addWidget(popup)
        
        # Add theme toggle button at the end of the banner
        if theme_button is not None:
            # Reuse the existing theme button
            self.theme_button = theme_button
            self.banner_layout.addWidget(self.theme_button)
        elif not hasattr(self, 'theme_button') or self.theme_button is None:
            # Create a new theme button if needed
            self.theme_button = QToolButton()
            self.theme_button.setCursor(Qt.CursorShape.PointingHandCursor)
            self.theme_button.setMinimumSize(60, 70)
            self.theme_button.setMaximumSize(60, 70)
            self.update_theme_button()  # Set text and style
            self.theme_button.clicked.connect(self.request_theme_toggle)
            self.banner_layout.addWidget(self.theme_button)
    
    def eventFilter(self, obj, event):
        """Global event filter for event behaviors"""
        # Early return to prevent extra processing if no popup is active
        if not hasattr(self, 'active_popup') or self.active_popup is None:
            return super().eventFilter(obj, event)
            
        # Handle mouse press events to close popups when clicking outside
        try:
            if event.type() == event.Type.MouseButtonPress:
                # Check if popup is still valid (not deleted)
                if not hasattr(self.active_popup, 'geometry'):
                    # Popup has been deleted, clean up references
                    self.active_popup = None
                    self.active_popup_owner = None
                    return False  # Let the event propagate
                
                # Get the position of the mouse click
                mouse_pos = event.globalPosition().toPoint()
                
                try:
                    # Check if the click is outside the active popup
                    if not self.active_popup.geometry().contains(
                            self.active_popup.mapFromGlobal(mouse_pos)):
                        # Check if owner is still valid
                        if self.active_popup_owner and hasattr(self.active_popup_owner, 'geometry'):
                            try:
                                # Check if click is on the button that owns the popup
                                if self.active_popup_owner.geometry().contains(
                                        self.active_popup_owner.mapFromGlobal(mouse_pos)):
                                    return False  # Let the button handle the event
                            except RuntimeError:
                                # Owner was deleted
                                self.active_popup_owner = None
                                
                        # Don't close the popup if we're clicking on something that might be inside it
                        # That includes buttons or other interactive elements
                        try:
                            widget_at_pos = QApplication.widgetAt(mouse_pos)
                            if widget_at_pos and isinstance(widget_at_pos, (QPushButton, QToolButton)):
                                # If it's a button, let it handle the event without closing popup
                                return False
                        except Exception:
                            pass
                        
                        # Otherwise close the popup
                        self.close_active_popup()
                        return False  # Let the event propagate
                except RuntimeError:
                    # Popup was deleted during operation
                    self.active_popup = None
                    self.active_popup_owner = None
                
        except Exception as e:
            print(f"Error in event filter: {e}")
            # Reset popup state to avoid further errors
            self.active_popup = None
            self.active_popup_owner = None
        
        # Let event be handled normally
        return super().eventFilter(obj, event)
    
    def create_favorites_popup(self):
        """Create a popup for favorites"""
        # If a favorites popup already exists, clean it up first
        if hasattr(self, 'favorites_popup') and self.favorites_popup is not None:
            try:
                # If this is the active popup, clear that reference
                if self.active_popup == self.favorites_popup:
                    self.active_popup = None
                    self.active_popup_owner = None
                
                # Remove from layout
                self.content_layout.removeWidget(self.favorites_popup)
                self.favorites_popup.deleteLater()
            except RuntimeError:
                # Skip if already deleted
                pass
        
        bg_color = "#ffffff" if not self.dark_mode else "#2d2d2d"
        popup = RoundedWidget(self, bg_color=bg_color)
        
        # Create layout
        layout = QVBoxLayout(popup)
        layout.setContentsMargins(5, 5, 5, 5)
        layout.setSpacing(0)
        
        # Title
        title_widget = QWidget()
        title_layout = QHBoxLayout(title_widget)
        title_layout.setContentsMargins(10, 5, 10, 5)
        
        title_label = QLabel("❤️ Favorites")  # Heart emoji
        title_label.setStyleSheet("font-size: 14px; font-weight: bold;")
        
        title_layout.addWidget(title_label)
        title_layout.addStretch(1)
        
        # Close button
        close_button = QToolButton()
        close_button.setText("×")
        close_button.setToolTip("Close")
        close_button.setCursor(Qt.CursorShape.PointingHandCursor)
        close_button.setStyleSheet("""
            QToolButton {
                font-size: 20px;
                background: transparent;
                border: none;
            }
        """)
        close_button.clicked.connect(self.close_active_popup)
        title_layout.addWidget(close_button)
        
        layout.addWidget(title_widget)
        
        # Add separator
        separator = QFrame()
        separator.setFrameShape(QFrame.Shape.HLine)
        separator.setFrameShadow(QFrame.Shadow.Sunken)
        separator.setStyleSheet("background-color: #dddddd;" if not self.dark_mode else "background-color: #444444;")
        layout.addWidget(separator)
        
        # Scroll area for favorites - with auto-scrollbar behavior
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        
        # Container for favorites
        favorites_widget = QWidget()
        favorites_layout = QVBoxLayout(favorites_widget)
        favorites_layout.setContentsMargins(5, 5, 5, 5)
        favorites_layout.setSpacing(2)
        favorites_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        
        # Check if there are any favorites
        has_favorite_items = False
        
        # Add management buttons
        # Store reference to self to use in lambda
        self_ref = self  # Create reference to self for lambda closure
        
        # Add rearrange button if there are favorites
        if self.store.favorite_commands:
            rearrange_button = QPushButton("Rearrange Favorites")
            rearrange_button.setCursor(Qt.CursorShape.PointingHandCursor)
            rearrange_button.setStyleSheet("""
                QPushButton {
                    background-color: #f5f5f5;
                    color: #333;
                    border: 1px solid #ddd;
                    padding: 8px 12px;
                    border-radius: 4px;
                    font-weight: bold;
                }
                QPushButton:hover {
                    background-color: #e5e5e5;
                    border-color: #ccc;
                }
            """)
            # Use lambda to explicitly close the popup first
            rearrange_button.clicked.connect(lambda checked=False: (
                self_ref.close_active_popup(),
                QTimer.singleShot(100, self_ref.show_favorites_manager)
            ))
            favorites_layout.addWidget(rearrange_button)
        
        # Always add Show/Hide Plugins button
        showhide_button = QPushButton("Show/Hide Plugins")
        showhide_button.setCursor(Qt.CursorShape.PointingHandCursor)
        showhide_button.setStyleSheet("""
            QPushButton {
                background-color: #f5f5f5;
                color: #333;
                border: 1px solid #ddd;
                padding: 8px 12px;
                border-radius: 4px;
                font-weight: bold;
                margin-top: 5px;
            }
            QPushButton:hover {
                background-color: #e5e5e5;
                border-color: #ccc;
            }
        """)
        # Use lambda to explicitly close the popup first
        showhide_button.clicked.connect(lambda checked=False: (
            self_ref.close_active_popup(),
            QTimer.singleShot(100, self_ref.show_plugins_manager)
        ))
        favorites_layout.addWidget(showhide_button)
        
        for favorite in self.store.favorite_commands:
            for plugin in self.store.plugins:
                if plugin.id == favorite.plugin_id:
                    for cmd in plugin.commands:
                        if cmd.js_function == favorite.command_function:
                            # Create a modified copy of the command with the display name
                            import copy
                            modified_cmd = copy.copy(cmd)
                            
                            # Use display_name (may be custom) instead of original command_name
                            print(f"Using display name for favorite: {favorite.display_name}")
                            modified_cmd.name = favorite.display_name
                            
                            # Create a special version of the command item that shows it's using a custom name
                            item = CommandItem(plugin, modified_cmd, popup)
                            
                            # If the display name is different from original, add a visual indicator
                            if favorite.display_name != favorite.command_name:
                                # Mark this as a custom name
                                item.custom_name = True
                                # Add an indicator to show the original command name
                                item.setToolTip(f"Custom name for: {favorite.command_name}")
                                
                                # Add a visual indicator that this is a custom name
                                item.name_label.setText(f"<b>{favorite.display_name}</b> <span style='color: #4a90e2; font-size: 11px;'>(custom)</span>")
                                
                            favorites_layout.addWidget(item)
                            has_favorite_items = True
                            break
        
        # Add an empty state message if no favorite items
        if not has_favorite_items:
            empty_label = QLabel("No favorite commands")
            empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            empty_label.setStyleSheet("color: #888888; padding: 20px;")
            favorites_layout.addWidget(empty_label)
        
        scroll_area.setWidget(favorites_widget)
        layout.addWidget(scroll_area)
        
        # Set minimum width but allow expansion for long command names
        popup.setMinimumWidth(400)
        popup.setMaximumWidth(600)  # Allow wider popups for long command names
        
        # We'll let the height be determined by content, with reasonable limits
        popup.setMinimumHeight(100)
        popup.setMaximumHeight(600)  # Larger max height to prevent scrolling in most cases
        
        # Hide initially
        popup.setVisible(False)
        
        # Add to layout
        self.content_layout.addWidget(popup)
        self.favorites_popup = popup
        
        # Set popup to be frameless but NOT a popup window type initially
        # This ensures it's correctly parented and doesn't create input issues
        popup.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.NoDropShadowWindowHint)
        
        # Ensure it doesn't steal focus
        popup.setAttribute(Qt.WidgetAttribute.WA_ShowWithoutActivating)
    
    def show_favorites_popup(self, button):
        """Show the favorites popup"""
        try:
            # Track the state before we close
            previous_popup = self.active_popup
            previous_owner = self.active_popup_owner
            
            # Always close any existing popup first
            if self.active_popup is not None:
                self.close_active_popup()
                
            # Process events to ensure UI is updated
            QApplication.processEvents()
            
            # Don't reopen if we just closed the same popup (toggle behavior)
            if previous_popup is not None and previous_owner == button:
                return
                
            # Ensure we have a valid favorites popup
            if not hasattr(self, 'favorites_popup') or self.favorites_popup is None or isdeleted(self.favorites_popup):
                # Recreate if missing or deleted
                self.create_favorites_popup()
                
            # Make popup a floating window again since our fix caused other issues
            # Set window type to Tool to prevent focus stealing while keeping it visible
            self.favorites_popup.setWindowFlags(Qt.WindowType.Tool | Qt.WindowType.FramelessWindowHint | Qt.WindowType.NoDropShadowWindowHint)
            self.favorites_popup.setAttribute(Qt.WidgetAttribute.WA_ShowWithoutActivating)
            
            # Ensure popup remains visible when buttons inside it are clicked
            for child in self.favorites_popup.findChildren(QPushButton):
                child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
            for child in self.favorites_popup.findChildren(QToolButton):
                child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
            
            # Position popup centered under the favorites button in screen coordinates
            button_pos = button.mapToGlobal(button.rect().bottomLeft())
            
            # Center popup under button
            x = int(button_pos.x() - (self.favorites_popup.width() - button.width()) / 2)
            y = int(button_pos.y() + 5)
            
            # Get screen geometry to ensure popup doesn't go off screen
            screen = QApplication.primaryScreen()
            screen_geo = screen.availableGeometry()
            
            # Ensure popup doesn't go off screen
            x = max(screen_geo.left() + 10, min(x, screen_geo.right() - self.favorites_popup.width() - 10))
            
            # Move popup to position and show it
            self.favorites_popup.move(x, y)
            self.favorites_popup.setVisible(True)
            self.active_popup = self.favorites_popup
            self.active_popup_owner = button
        except Exception as e:
            print(f"Error showing favorites popup: {e}")
            # Clean up state if we failed
            self.active_popup = None
            self.active_popup_owner = None
            
    def show_favorites_manager(self):
        """Open a separate window for managing favorites"""
        try:
            # First close any existing popup
            if self.active_popup is not None:
                self.close_active_popup()
                
            # Process events to ensure the popup is fully closed
            QApplication.processEvents()
            
            # Try to import the favorites manager window
            from ui.favorites_view import FavoritesManagerWindow
            
            print("Opening favorites manager window...")
            
            # Create an independent window (no parent)
            # This avoids event filter issues with modal dialogs
            window = FavoritesManagerWindow(None)
            window.setWindowTitle("Manage Favorites")
            window.show()
            window.activateWindow()  # Make sure it gets focus
            window.raise_()  # Bring it to the front
            
            # Process events to ensure window is properly shown
            QApplication.processEvents()
            
        except Exception as e:
            print(f"Error showing favorites manager: {e}")
    
    def show_plugins_manager(self):
        """Open a separate window for managing plugin visibility"""
        try:
            # First close any existing popup
            if self.active_popup is not None:
                self.close_active_popup()
                
            # Process events to ensure the popup is fully closed
            QApplication.processEvents()
            
            # Import the plugins manager window
            from ui.plugins_visibility_view import PluginsVisibilityManagerWindow
            
            print("Opening plugins visibility manager window...")
            
            # Create an independent window (no parent)
            window = PluginsVisibilityManagerWindow(None)
            window.setWindowTitle("Show/Hide Plugins")
            window.show()
            window.activateWindow()  # Make sure it gets focus
            window.raise_()  # Bring it to the front
            
            # Process events to ensure window is properly shown
            QApplication.processEvents()
            
        except Exception as e:
            print(f"Error showing plugins visibility manager: {e}")
    
    def show_plugin_popup(self, plugin, from_hover=False):
        """Show popup for a plugin"""
        try:
            # Check if plugin ID exists in popups dictionary
            if plugin.id not in self.plugin_popups:
                return
                
            popup = self.plugin_popups[plugin.id]
            
            # Check if the popup is valid
            if popup is None or isdeleted(popup):
                # Recreate if missing or deleted
                popup = PluginPopup(plugin, self)
                self.plugin_popups[plugin.id] = popup
                self.content_layout.addWidget(popup)
            
            # Check if plugin ID exists in buttons dictionary
            if plugin.id not in self.plugin_buttons:
                return
                
            button = self.plugin_buttons[plugin.id]
            
            # Track the state before we close
            previous_popup = self.active_popup
            previous_owner = self.active_popup_owner
            
            # Always close any existing popup first
            if self.active_popup is not None:
                self.close_active_popup()
                
            # Process events to ensure UI is updated
            QApplication.processEvents()
            
            # Don't reopen if we just closed the same popup (toggle behavior)
            if previous_popup is not None and previous_owner == button:
                return
                
            # Make popup a floating window again since our fix caused other issues
            # Set window type to Tool to prevent focus stealing while keeping it visible
            popup.setWindowFlags(Qt.WindowType.Tool | Qt.WindowType.FramelessWindowHint | Qt.WindowType.NoDropShadowWindowHint)
            popup.setAttribute(Qt.WidgetAttribute.WA_ShowWithoutActivating)
            
            # Ensure popup remains visible when buttons inside it are clicked
            for child in popup.findChildren(QPushButton):
                child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
            for child in popup.findChildren(QToolButton):
                child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
            
            # Get global position of button
            button_pos = button.mapToGlobal(button.rect().bottomLeft())
            
            # Center popup under button
            x = int(button_pos.x() - (popup.width() - button.width()) / 2)
            y = int(button_pos.y() + 5)
            
            # Get screen geometry to ensure popup doesn't go off screen
            screen = QApplication.primaryScreen()
            screen_geo = screen.availableGeometry()
            
            # Ensure popup doesn't go off screen
            x = max(screen_geo.left() + 10, min(x, screen_geo.right() - popup.width() - 10))
            
            # Move popup to position and show it
            popup.move(x, y)
            popup.setVisible(True)
            
            # Record the active popup and its owner
            self.active_popup = popup
            self.active_popup_owner = button
            
        except Exception as e:
            print(f"Error showing plugin popup: {e}")
            # Clean up state if we failed
            self.active_popup = None
            self.active_popup_owner = None
    
    def popup_enter_event(self, popup, event):
        """Handle mouse entering popup"""
        # For click-based interaction, we don't need to do anything on mouse enter
        pass
    
    def popup_leave_event(self, popup, event):
        """Handle mouse leaving popup"""
        # For click-based interaction, we don't automatically close on mouse leave
        # The popup will stay visible until the user clicks elsewhere or clicks the close button
        pass
    
    def close_active_popup(self):
        """Close the active popup"""
        try:
            if self.active_popup:
                # Store reference to what we're about to close
                popup_to_close = self.active_popup
                old_owner = self.active_popup_owner
                
                # Clear the active popup references first
                self.active_popup = None
                self.active_popup_owner = None
                
                # Now hide the popup (only if it's still valid)
                if popup_to_close and not isdeleted(popup_to_close) and hasattr(popup_to_close, 'setVisible'):
                    try:
                        popup_to_close.setVisible(False)
                    except RuntimeError:
                        # Widget might have been deleted during this call
                        pass
                
                # Process events to ensure UI updates
                QApplication.processEvents()
        except Exception as e:
            print(f"Error closing popup: {e}")
            # Reset state to avoid future errors
            self.active_popup = None
            self.active_popup_owner = None
    
    def on_favorites_updated(self):
        """Handle favorites updated signal"""
        try:
            # Close the active popup if it's the favorites popup
            if self.active_popup is not None and hasattr(self, 'favorites_popup') and self.active_popup == self.favorites_popup:
                self.close_active_popup()
            
            # Recreate the favorites popup with updated content
            self.create_favorites_popup()
            
            # Update the entire banner to ensure favorites button is shown/hidden properly
            self.update_plugins()
        except Exception as e:
            print(f"Error updating favorites: {e}")
    
    def set_dark_mode(self, dark_mode):
        """Update UI for dark/light mode"""
        self.dark_mode = dark_mode
        
        # Update banner background color
        if dark_mode:
            self.banner.setStyleSheet("background-color: #2d2d2d;")
        else:
            self.banner.setStyleSheet("background-color: #FDF6E3;")  # Keep cream color in light mode
        
        # Update plugin buttons
        for button in self.plugin_buttons.values():
            try:
                button.update_style(dark_mode)
            except RuntimeError:
                # Skip deleted objects
                pass
        
        # Update favorite button if it exists
        if hasattr(self, 'favorites_button') and self.favorites_button is not None:
            try:
                if dark_mode:
                    self.favorites_button.setStyleSheet("""
                        QToolButton {
                            font-size: 36px;
                            background: transparent;
                            border: none;
                            color: #f0f0f0;
                            padding: 5px;
                        }
                        QToolButton:hover {
                            background-color: rgba(255, 255, 255, 0.1);
                            border-radius: 10px;
                        }
                        QToolButton:pressed {
                            background-color: rgba(255, 255, 255, 0.15);
                            border-radius: 10px;
                        }
                    """)
                else:
                    self.favorites_button.setStyleSheet("""
                        QToolButton {
                            font-size: 36px;
                            background: transparent;
                            border: none;
                            color: #333333;
                            padding: 5px;
                        }
                        QToolButton:hover {
                            background-color: rgba(0, 0, 0, 0.05);
                            border-radius: 10px;
                        }
                        QToolButton:pressed {
                            background-color: rgba(0, 0, 0, 0.1);
                            border-radius: 10px;
                        }
                    """)
            except RuntimeError:
                # Skip deleted objects
                pass
        
        # Update theme button with error handling
        try:
            if hasattr(self, 'theme_button') and self.theme_button is not None:
                if self.dark_mode:
                    self.theme_button.setText("☀️")
                    self.theme_button.setToolTip("Switch to Light Mode")
                    self.theme_button.setStyleSheet("""
                        QToolButton {
                            font-size: 24px;
                            background: transparent;
                            border: none;
                            color: #f0f0f0;
                            padding: 5px;
                        }
                        QToolButton:hover {
                            background-color: rgba(255, 255, 255, 0.1);
                            border-radius: 10px;
                        }
                        QToolButton:pressed {
                            background-color: rgba(255, 255, 255, 0.15);
                            border-radius: 10px;
                        }
                    """)
                else:
                    self.theme_button.setText("🌙")
                    self.theme_button.setToolTip("Switch to Dark Mode")
                    self.theme_button.setStyleSheet("""
                        QToolButton {
                            font-size: 24px;
                            background: transparent;
                            border: none;
                            color: #333333;
                            padding: 5px;
                        }
                        QToolButton:hover {
                            background-color: rgba(0, 0, 0, 0.05);
                            border-radius: 10px;
                        }
                        QToolButton:pressed {
                            background-color: rgba(0, 0, 0, 0.1);
                            border-radius: 10px;
                        }
                    """)
        except RuntimeError:
            # Skip if button has been deleted
            pass
        
        # Update popups with error handling
        for popup_id, popup in list(self.plugin_popups.items()):
            try:
                popup.bg_color = "#ffffff" if not dark_mode else "#2d2d2d"
                popup.update()
            except RuntimeError:
                # Remove deleted popups
                self.plugin_popups.pop(popup_id, None)
        
        # Update favorites popup if it exists
        if hasattr(self, 'favorites_popup') and self.favorites_popup is not None:
            try:
                self.favorites_popup.bg_color = "#ffffff" if not dark_mode else "#2d2d2d"
                self.favorites_popup.update()
            except RuntimeError:
                # Skip if popup has been deleted
                pass