import subprocess
import time
from PyQt6.QtCore import (
    Qt, QMimeData, QPoint, QTimer, QObject, QEvent, QPropertyAnimation,
    QEasingCurve, QAbstractAnimation
)
from PyQt6.QtGui import QDrag, QIcon, QColor
from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QScrollArea, QLabel, 
    QPushButton, QFrame, QHBoxLayout, QApplication,
    QToolButton, QMainWindow, QLineEdit,
    QGraphicsOpacityEffect, QGraphicsDropShadowEffect
)

from models import PluginStore


class FavoriteCommandItem(QWidget):
    """Widget representing a favorite command"""
    
    def __init__(self, favorite, index, parent=None):
        super().__init__(parent)
        self.favorite = favorite
        self.favorites_index = index  # Store index for reordering
        self.store = PluginStore.get_instance()
        self.drag_start_position = None
        self.parent_view = parent
        self.editing = False
        
        # Create overall frame with shadow effect
        self.setObjectName("favoriteItem")
        
        # Add a subtle shadow effect
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(8)
        shadow.setColor(QColor(0, 0, 0, 40))
        shadow.setOffset(0, 2)
        self.setGraphicsEffect(shadow)
        
        # Create layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(12, 12, 12, 12)
        layout.setSpacing(12)
        
        # Add drag handle button with improved styling
        self.drag_handle = QToolButton()
        self.drag_handle.setText("☰")  # hamburger/grab icon
        self.drag_handle.setToolTip("Drag to reorder")
        self.drag_handle.setCursor(Qt.CursorShape.OpenHandCursor)
        self.drag_handle.setFixedSize(40, 40)
        self.drag_handle.setStyleSheet("""
            QToolButton {
                color: #555;
                background-color: #f0f0f0;
                border: 1px solid #e0e0e0;
                border-radius: 6px;
                font-size: 20px;
                font-weight: bold;
                padding: 5px;
            }
            QToolButton:hover {
                color: #333;
                background-color: #e8e8e8;
                border: 1px solid #d0d0d0;
            }
        """)
        layout.addWidget(self.drag_handle)
        
        # Command info - wrapped in a widget for editing mode
        self.info_container = QWidget()
        self.info_container.setStyleSheet("background: transparent;")
        info_layout = QVBoxLayout(self.info_container)
        info_layout.setContentsMargins(0, 0, 0, 0)
        info_layout.setSpacing(4)
        
        # Replace label with always-editable line edit
        self.name_edit = QLineEdit(favorite.display_name)
        self.name_edit.setMinimumHeight(36)
        self.name_edit.setStyleSheet("""
            QLineEdit {
                padding: 8px 12px;
                border: 1px solid #e0e0e0;
                border-radius: 5px;
                background-color: white;
                font-size: 14px;
                min-height: 36px;
                font-weight: bold;
                color: #333;
            }
            QLineEdit:focus {
                border: 1px solid #4a90e2;
            }
            QLineEdit:hover:not(:focus) {
                background-color: #f8f8f8;
            }
        """)
        info_layout.addWidget(self.name_edit)
        
        # Connect editing finished signal to save the name
        self.name_edit.editingFinished.connect(self.save_rename)
        
        # Plugin name with pill background
        plugin_container = QWidget()
        plugin_container.setStyleSheet("background: transparent;")
        plugin_layout = QHBoxLayout(plugin_container)
        plugin_layout.setContentsMargins(0, 0, 0, 0)
        plugin_layout.setSpacing(0)
        plugin_layout.setAlignment(Qt.AlignmentFlag.AlignLeft)
        
        self.plugin_label = QLabel(favorite.plugin_name)
        self.plugin_label.setStyleSheet("""
            color: #555;
            background-color: #f0f0f0;
            padding: 2px 8px;
            border-radius: 10px;
            font-size: 12px;
        """)
        plugin_layout.addWidget(self.plugin_label)
        plugin_layout.addStretch()
        
        info_layout.addWidget(plugin_container)
        
        # We now have the edit field directly integrated into the UI, no need for separate edit controls
        
        layout.addWidget(self.info_container, 1)
        
        # Action buttons container with improved spacing
        button_container = QWidget()
        button_container.setStyleSheet("background: transparent;")
        button_layout = QHBoxLayout(button_container)
        button_layout.setContentsMargins(0, 0, 0, 0)
        button_layout.setSpacing(8)
        
        # No longer need a separate edit button, since editing is inline
        button_layout.addSpacing(0)
        
        # Remove button with improved styling
        self.remove_button = QToolButton()
        self.remove_button.setText("❤️")  # Heart emoji
        self.remove_button.setToolTip("Remove from favorites")
        self.remove_button.setFixedSize(36, 36)
        self.remove_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.remove_button.setStyleSheet("""
            QToolButton {
                background-color: #f5f5f5;
                border: 1px solid #e0e0e0;
                border-radius: 18px;
                font-size: 16px;
            }
            QToolButton:hover {
                background-color: #fff0e0;
                border: 1px solid #ff6b6b;
            }
        """)
        self.remove_button.clicked.connect(lambda checked=False: self.remove_favorite())
        button_layout.addWidget(self.remove_button)
        
        # Run button with improved styling
        self.run_button = QPushButton("Run")
        self.run_button.setFixedSize(70, 36)
        self.run_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.run_button.setStyleSheet("""
            QPushButton {
                background-color: #4a90e2;
                color: white;
                border-radius: 5px;
                font-weight: bold;
                font-size: 13px;
                border: none;
            }
            QPushButton:hover {
                background-color: #3a80d2;
            }
            QPushButton:pressed {
                background-color: #2a70c2;
            }
        """)
        self.run_button.clicked.connect(lambda checked=False: self.run_command())
        button_layout.addWidget(self.run_button)
        
        layout.addWidget(button_container)
        
        # Set widget properties
        self.setFixedHeight(layout.sizeHint().height() + 10)  # Slightly taller for edit mode
        self.setAutoFillBackground(True)
        
        # Default background color and border styling
        self.setStyleSheet("""
            #favoriteItem {
                background-color: white;
                border: 1px solid #e0e0e0;
                border-radius: 8px;
            }
            #favoriteItem:hover {
                background-color: #f8f8f8;
                border: 1px solid #d0d0d0;
            }
        """)
        
        # Enable mouse tracking for drag and drop
        self.setMouseTracking(True)
        self.drag_handle.setMouseTracking(True)
        
        # Install event filter on drag handle to track mouse events
        self.drag_handle.installEventFilter(self)
    
    def save_rename(self):
        """Save the renamed command when editing is finished"""
        new_name = self.name_edit.text().strip()
        
        # If empty, revert to original name
        if not new_name:
            self.name_edit.setText(self.favorite.display_name)
            return
            
        # Only save if the name actually changed
        if new_name != self.favorite.display_name:
            print(f"Saving renamed favorite: '{self.favorite.display_name}' -> '{new_name}'")
            old_name = self.favorite.display_name
            
            # Update the favorites list in the store
            self.store.update_favorite_display_name(self.favorites_index, new_name)
            
            # Also update our local copy to match
            self.favorite.display_name = new_name
            
            # Manually update the banner favorites by forcing a close and reopen of the popup
            # This avoids the need to emit the favorites_updated signal
            if hasattr(self, 'parent_view') and self.parent_view and hasattr(self.parent_view, 'parent'):
                main_window = self.parent_view.parent
                if hasattr(main_window, 'main_window') and main_window.main_window:
                    if hasattr(main_window.main_window, 'banner_view'):
                        banner = main_window.main_window.banner_view
                        # Close the popup if it's open
                        if hasattr(banner, 'close_active_popup'):
                            banner.close_active_popup()
                            # Create the favorites popup again to refresh it
                            if hasattr(banner, 'create_favorites_popup'):
                                banner.create_favorites_popup()
        
    def eventFilter(self, obj, event):
        """Event filter to handle drag initiation from the drag handle"""
        if obj == self.drag_handle:
            if event.type() == event.Type.MouseButtonPress and event.button() == Qt.MouseButton.LeftButton:
                self.drag_start_position = event.position()
                # Change cursor to closed hand
                try:
                    self.drag_handle.setCursor(Qt.CursorShape.ClosedHandCursor)
                except RuntimeError:
                    # Handle case where widget has been deleted
                    return False
                return True
            elif event.type() == event.Type.MouseMove and event.buttons() & Qt.MouseButton.LeftButton:
                # Check if we're dragging far enough to start a drag operation
                if hasattr(self, 'drag_start_position') and self.drag_start_position is not None:
                    try:
                        distance = (event.position() - self.drag_start_position).manhattanLength()
                        if distance >= QApplication.startDragDistance():
                            # Don't reset cursor here - let it reset naturally when the widget is recreated
                            self.startDrag()
                            return True
                    except RuntimeError:
                        # Handle case where widget has been deleted
                        return False
            elif event.type() == event.Type.MouseButtonRelease:
                # Reset cursor on mouse release
                try:
                    self.drag_handle.setCursor(Qt.CursorShape.OpenHandCursor)
                except RuntimeError:
                    # Handle case where widget has been deleted
                    return False
                
        return super().eventFilter(obj, event)
    
    def startDrag(self):
        """Start drag operation for reordering"""
        try:
            if not hasattr(self, 'parent_view') or self.parent_view is None:
                return
                
            drag = QDrag(self)
            mimedata = QMimeData()
            
            # Store the index of this item for the drop handler
            mimedata.setData("application/x-favorite-index", str(self.favorites_index).encode())
            drag.setMimeData(mimedata)
            
            # Create a visual representation of what's being dragged
            try:
                pixmap = self.grab()
                drag.setPixmap(pixmap)
                drag.setHotSpot(QPoint(pixmap.width()//2, pixmap.height()//2))
            except RuntimeError:
                # If grab fails, use a simple placeholder
                pass
            
            # Execute the drag and don't depend on the result
            try:
                drag.exec(Qt.DropAction.MoveAction)
            except RuntimeError:
                # Handle case where drag operation fails
                pass
        except Exception as e:
            print(f"Error in drag operation: {e}")
    
    def remove_favorite(self):
        """Remove from favorites"""
        try:
            # Ensure necessary imports are available
            import subprocess
            import time
            
            # Show wait cursor
            QApplication.instance().setOverrideCursor(Qt.CursorShape.WaitCursor)
            
            # Look for the plugin and command
            for plugin in self.store.plugins:
                if plugin.id == self.favorite.plugin_id:
                    for cmd in plugin.commands:
                        if cmd.js_function == self.favorite.command_function:
                            self.store.toggle_favorite(plugin, cmd)
                            QApplication.instance().restoreOverrideCursor()
                            return
                            
            # Restore cursor
            QApplication.instance().restoreOverrideCursor()
        except Exception as e:
            # Make sure cursor is restored
            try:
                QApplication.instance().restoreOverrideCursor()
            except:
                pass
            print(f"Error removing favorite: {e}")
    
    def run_command(self):
        """Run the favorite command"""
        try:
            # Direct imports to ensure they're available
            import subprocess
            import time
            import os
            
            # Show wait cursor
            QApplication.instance().setOverrideCursor(Qt.CursorShape.WaitCursor)
            
            # Look for the plugin and command
            for plugin in self.store.plugins:
                if plugin.id == self.favorite.plugin_id:
                    for cmd in plugin.commands:
                        if cmd.js_function == self.favorite.command_function:
                            try:
                                self.store.execute_command(plugin, cmd)
                                QApplication.instance().restoreOverrideCursor()
                                return
                            except Exception as store_error:
                                print(f"Error executing through store: {store_error}")
                                # Continue to direct execution
                                break
            
            # If plugin/command not found or store execution failed, try direct URL using multiple methods
            plugin_id = self.favorite.plugin_id.replace(" ", "%20")
            
            # IMPORTANT: Use the original command_name for execution, not the display_name
            # This ensures we're calling the actual plugin command, even if it's been renamed
            command_name = self.favorite.command_name.replace(" ", "%20")
            
            # First try directly opening NotePlan
            subprocess.run(["open", "-a", "NotePlan"], check=False)
            
            # Construct the URL with proper encoding
            url = f"noteplan://x-callback-url/runPlugin?pluginID={plugin_id}&command={command_name}"
            print(f"Running favorite command: {url} (display name: {self.favorite.display_name})")
            
            # Wait a moment for NotePlan to be ready
            time.sleep(0.3)
            
            # Method 1: Try direct open command (most reliable)
            try:
                subprocess.run(["open", url], check=False)
            except Exception as e:
                print(f"Method 1 failed: {e}")
                
            # Wait briefly between methods
            time.sleep(0.2)
                
            # Method 2: Use AppleScript as fallback
            try:
                applescript = f'tell application "NotePlan" to activate\ntell application "System Events" to open location "{url}"'
                subprocess.run(["osascript", "-e", applescript], check=False)
            except Exception as e:
                print(f"Method 2 failed: {e}")
                
            # Method 3: Try one more AppleScript variant as last resort
            try:
                applescript2 = f'do shell script "open \\"{url}\\""'
                subprocess.run(["osascript", "-e", applescript2], check=False)
            except Exception as e:
                print(f"Method 3 failed: {e}")
                
            # Restore cursor
            QApplication.instance().restoreOverrideCursor()
        except Exception as e:
            # Make sure cursor is restored
            try:
                QApplication.instance().restoreOverrideCursor()
            except:
                pass
            print(f"Error running command: {e}")


# Global reference to keep manager window alive
_manager_window_ref = None

def _clear_global_reference():
    """Helper function to clear the global window reference safely"""
    global _manager_window_ref
    print("Clearing global window reference")
    _manager_window_ref = None

class WindowCloseFilter(QObject):
    """Event filter to handle window closing and cleanup"""
    
    def eventFilter(self, obj, event):
        """Filter window close events"""
        if event.type() == QEvent.Type.Close:
            # Print message on close
            print("Favorites manager window closing...")
            
            # Allow the event to propagate
            return False
        
        # Let other events pass through
        return super().eventFilter(obj, event)


class FavoritesManagerWindow(QMainWindow):
    """Separate window for managing favorites with drag and drop reordering"""
    
    def __init__(self, parent=None):
        global _manager_window_ref
        
        super().__init__(None)  # Create as a separate independent window
        self.setWindowTitle("Manage Favorites")
        self.setMinimumSize(600, 450)
        
        # Set up window flags to ensure it behaves properly
        self.setWindowFlags(
            Qt.WindowType.Window |
            Qt.WindowType.WindowTitleHint |
            Qt.WindowType.WindowCloseButtonHint |
            Qt.WindowType.WindowStaysOnTopHint  # Make it stay on top
        )
        
        # Install event filter for window close
        self.event_filter = WindowCloseFilter()
        self.installEventFilter(self.event_filter)
        
        # Keep a global reference to prevent garbage collection
        _manager_window_ref = self
        
        self.store = PluginStore.get_instance()
        self.parent_widget = parent
        
        # Create central widget with background color
        central_widget = QWidget()
        central_widget.setStyleSheet("background-color: #f9f9f9;")
        self.setCentralWidget(central_widget)
        
        # Main layout
        layout = QVBoxLayout(central_widget)
        layout.setContentsMargins(20, 20, 20, 20)
        layout.setSpacing(15)
        
        # Header with title and instructions
        header_widget = QWidget()
        header_widget.setStyleSheet("background-color: #4a90e2; border-radius: 8px;")
        header_layout = QVBoxLayout(header_widget)
        header_layout.setContentsMargins(15, 15, 15, 15)
        
        title = QLabel("Manage Your Favorites")
        title.setStyleSheet("font-size: 18px; font-weight: bold; color: white;")
        title.setAlignment(Qt.AlignmentFlag.AlignCenter)
        header_layout.addWidget(title)
        
        instructions = QLabel("Drag the ☰ handles to reorder · Click ✎ to rename · Click ★ to remove")
        instructions.setStyleSheet("font-size: 13px; color: rgba(255, 255, 255, 0.9); margin-top: 5px;")
        instructions.setAlignment(Qt.AlignmentFlag.AlignCenter)
        header_layout.addWidget(instructions)
        
        layout.addWidget(header_widget)
        
        # Add the favorites view with a border and shadow
        favorites_container = QFrame()
        favorites_container.setFrameShape(QFrame.Shape.StyledPanel)
        favorites_container.setStyleSheet("""
            QFrame {
                background-color: white;
                border: 1px solid #ddd;
                border-radius: 8px;
            }
        """)
        
        # Add drop shadow effect
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(15)
        shadow.setColor(QColor(0, 0, 0, 30))
        shadow.setOffset(0, 2)
        favorites_container.setGraphicsEffect(shadow)
        
        favorites_layout = QVBoxLayout(favorites_container)
        favorites_layout.setContentsMargins(2, 2, 2, 2)
        
        self.favorites_view = FavoritesView(self)
        favorites_layout.addWidget(self.favorites_view)
        
        layout.addWidget(favorites_container, 1)
        
        # Bottom buttons with improved styling
        buttons_container = QWidget()
        buttons_container.setStyleSheet("background-color: transparent;")
        buttons_layout = QHBoxLayout(buttons_container)
        buttons_layout.setContentsMargins(0, 5, 0, 0)
        
        # Empty spacer - removed "Changes are saved automatically" text
        buttons_layout.addStretch(0)
        
        buttons_layout.addStretch(1)
        
        close_button = QPushButton("Close")
        close_button.setCursor(Qt.CursorShape.PointingHandCursor)
        close_button.clicked.connect(self.delayed_close)
        close_button.setMinimumWidth(100)
        close_button.setStyleSheet("""
            QPushButton {
                background-color: #4a90e2;
                color: white;
                border: none;
                border-radius: 6px;
                padding: 10px 20px;
                font-weight: bold;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: #3a80d2;
            }
            QPushButton:pressed {
                background-color: #2a70c2;
            }
        """)
        buttons_layout.addWidget(close_button)
        
        layout.addWidget(buttons_container)
    
    def delayed_close(self):
        """Close the window after a small delay"""
        print("Close button clicked, will close window after delay...")
        # Schedule window close after a short delay
        QTimer.singleShot(200, self.close)
    
    def closeEvent(self, event):
        """Handle window close event"""
        global _manager_window_ref
        
        print("Favorites manager window closeEvent called")
        
        # Save any pending edits before closing
        self.save_all_pending_edits()
        
        # Force a refresh of the favorites menu in the main window
        self.store = PluginStore.get_instance()
        try:
            # Now we can safely emit the signal, since the window is closing anyway
            print("Emitting favorites_updated signal on window close")
            self.store.favorites_updated.emit()
        except Exception as e:
            print(f"Error updating favorites: {e}")
        
        # Accept the close event
        event.accept()
        
        # Clear the global reference after a delay
        # This prevents the window from being garbage collected too soon
        QTimer.singleShot(500, lambda: _clear_global_reference())
        
    def save_all_pending_edits(self):
        """Save any pending edits in all favorite items"""
        print("Saving all pending edits before closing")
        
        # Force focus change to trigger save on any currently edited field
        self.setFocus()


class FavoritesView(QWidget):
    """View displaying favorite commands"""
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.store = PluginStore.get_instance()
        self.item_widgets = []  # Keep track of item widgets
        
        # Create layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        
        # Improved scroll area with nicer styling
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        self.scroll_area.setStyleSheet("""
            QScrollArea {
                border: none;
                background-color: transparent;
            }
            QScrollBar:vertical {
                border: none;
                background: #f0f0f0;
                width: 8px;
                border-radius: 4px;
                margin: 0px;
            }
            QScrollBar::handle:vertical {
                background: #bbbbbb;
                min-height: 20px;
                border-radius: 4px;
            }
            QScrollBar::handle:vertical:hover {
                background: #999999;
            }
            QScrollBar::add-line:vertical, 
            QScrollBar::sub-line:vertical {
                height: 0px;
            }
            QScrollBar::add-page:vertical, 
            QScrollBar::sub-page:vertical {
                background: none;
            }
        """)
        
        # Container with improved styling
        self.container = QWidget()
        self.container.setObjectName("favoritesContainer")
        self.container.setStyleSheet("""
            #favoritesContainer {
                background-color: white;
                border-radius: 8px;
            }
        """)
        self.container.setAcceptDrops(True)  # Enable drops
        
        self.container_layout = QVBoxLayout(self.container)
        self.container_layout.setContentsMargins(15, 15, 15, 15)
        self.container_layout.setSpacing(12)
        self.container_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        
        # Empty state with improved styling
        self.empty_container = QWidget()
        empty_layout = QVBoxLayout(self.empty_container)
        empty_layout.setContentsMargins(20, 40, 20, 40)
        empty_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        empty_icon = QLabel("❤️")
        empty_icon.setStyleSheet("""
            font-size: 48px;
            margin-bottom: 15px;
        """)
        empty_icon.setAlignment(Qt.AlignmentFlag.AlignCenter)
        empty_layout.addWidget(empty_icon)
        
        self.empty_label = QLabel("No favorite commands yet")
        self.empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.empty_label.setStyleSheet("""
            font-size: 18px;
            color: #888;
            margin-bottom: 10px;
        """)
        empty_layout.addWidget(self.empty_label)
        
        empty_hint = QLabel("Add favorites from the main window by clicking the heart icon")
        empty_hint.setAlignment(Qt.AlignmentFlag.AlignCenter)
        empty_hint.setStyleSheet("""
            font-size: 14px;
            color: #aaa;
            margin-bottom: 20px;
        """)
        empty_layout.addWidget(empty_hint)
        
        self.container_layout.addWidget(self.empty_container)
        
        # Enable drops on container
        self.container.installEventFilter(self)
        
        self.scroll_area.setWidget(self.container)
        layout.addWidget(self.scroll_area)
        
        # Connect signals
        self.store.favorites_updated.connect(self.update_favorites_list)
        
        # Initialize
        self.update_favorites_list()
    
    def isDeleted(self, widget):
        """Check if a widget has been deleted"""
        try:
            # Try to access a property of the widget
            # This will raise RuntimeError if widget is deleted
            widget.objectName()
            return False
        except RuntimeError:
            return True
    
    def eventFilter(self, obj, event):
        """Event filter to handle drops in the container"""
        if obj == self.container:
            if event.type() == event.Type.DragEnter:
                # Accept the drag if it's our custom mime type
                if event.mimeData().hasFormat("application/x-favorite-index"):
                    event.accept()
                    return True
                else:
                    event.ignore()
                    
            elif event.type() == event.Type.DragMove:
                # Accept drag moves to enable drops
                if event.mimeData().hasFormat("application/x-favorite-index"):
                    event.accept()
                    
                    # Show visual indicator for drop position
                    if hasattr(self, 'drop_indicator'):
                        try:
                            if not self.isDeleted(self.drop_indicator):
                                self.drop_indicator.deleteLater()
                        except (RuntimeError, Exception) as e:
                            print(f"Error removing drop indicator: {e}")
                        
                        delattr(self, 'drop_indicator')
                    
                    # Create a visible drop indicator with enhanced styling
                    pos = event.position().toPoint()
                    target_idx = -1
                    
                    # Find the target position
                    for i, widget in enumerate(self.item_widgets):
                        try:
                            # Skip empty container and deleted widgets
                            if (hasattr(self, 'empty_container') and widget == self.empty_container) or self.isDeleted(widget):
                                continue
                                
                            widget_rect = widget.geometry()
                            
                            if widget_rect.contains(pos):
                                mid_y = widget_rect.y() + widget_rect.height() / 2
                                
                                # Create an improved horizontal indicator
                                indicator = QFrame(self.container)
                                indicator.setObjectName("dropIndicator")
                                
                                # Calculate position - above or below the item
                                if pos.y() < mid_y:
                                    # Position before current widget with improved styling
                                    indicator_y = widget_rect.y() - 6
                                    # Highlight the target item as well
                                    if hasattr(widget, 'setStyleSheet'):
                                        widget.setStyleSheet("""
                                            #favoriteItem {
                                                background-color: #f0f7ff;
                                                border: 1px solid #4a90e2;
                                                border-bottom: 1px solid #e0e0e0;
                                                border-radius: 8px;
                                            }
                                        """)
                                else:
                                    # Position after current widget with improved styling
                                    indicator_y = widget_rect.y() + widget_rect.height() + 1
                                    # Highlight the target item as well
                                    if hasattr(widget, 'setStyleSheet'):
                                        widget.setStyleSheet("""
                                            #favoriteItem {
                                                background-color: #f0f7ff;
                                                border: 1px solid #4a90e2; 
                                                border-top: 1px solid #e0e0e0;
                                                border-radius: 8px;
                                            }
                                        """)
                                
                                # Set full-width indicator with attractive styling
                                indicator.setGeometry(
                                    widget_rect.x(), indicator_y,
                                    widget_rect.width(), 4
                                )
                                
                                # Improved styling for the indicator
                                indicator.setStyleSheet("""
                                    #dropIndicator {
                                        background-color: #4a90e2;
                                        border-radius: 2px;
                                    }
                                """)
                                
                                indicator.show()
                                self.drop_indicator = indicator
                                break
                        except (RuntimeError, Exception) as e:
                            print(f"Error handling widget during drag: {e}")
                            continue
                    
                    return True
                else:
                    event.ignore()
                    
            elif event.type() == event.Type.DragLeave:
                # Remove drop indicator when drag leaves
                if hasattr(self, 'drop_indicator'):
                    try:
                        if not self.isDeleted(self.drop_indicator):
                            self.drop_indicator.deleteLater()
                    except (RuntimeError, Exception) as e:
                        print(f"Error removing drop indicator: {e}")
                    
                    delattr(self, 'drop_indicator')
                
                # Reset all items to default style
                for widget in self.item_widgets:
                    try:
                        if not self.isDeleted(widget) and hasattr(widget, 'setStyleSheet') and widget.objectName() == "favoriteItem":
                            widget.setStyleSheet("""
                                #favoriteItem {
                                    background-color: white;
                                    border: 1px solid #e0e0e0;
                                    border-radius: 8px;
                                }
                                #favoriteItem:hover {
                                    background-color: #f8f8f8;
                                    border: 1px solid #d0d0d0;
                                }
                            """)
                    except (RuntimeError, Exception) as e:
                        # Skip deleted widgets
                        continue
                    
            elif event.type() == event.Type.Drop:
                # Remove drop indicator
                if hasattr(self, 'drop_indicator'):
                    try:
                        if not self.isDeleted(self.drop_indicator):
                            self.drop_indicator.deleteLater()
                    except (RuntimeError, Exception) as e:
                        print(f"Error removing drop indicator: {e}")
                    
                    delattr(self, 'drop_indicator')
                
                # Reset all items to default style
                for widget in self.item_widgets:
                    try:
                        if not self.isDeleted(widget) and hasattr(widget, 'setStyleSheet') and widget.objectName() == "favoriteItem":
                            widget.setStyleSheet("""
                                #favoriteItem {
                                    background-color: white;
                                    border: 1px solid #e0e0e0;
                                    border-radius: 8px;
                                }
                                #favoriteItem:hover {
                                    background-color: #f8f8f8;
                                    border: 1px solid #d0d0d0;
                                }
                            """)
                    except (RuntimeError, Exception) as e:
                        # Skip deleted widgets
                        continue
                
                # Process the drop to reorder
                if event.mimeData().hasFormat("application/x-favorite-index"):
                    # Get the source index
                    source_idx = int(event.mimeData().data("application/x-favorite-index").data().decode())
                    
                    # Get the drop position and find which item we're dropping onto
                    pos = event.position().toPoint()
                    target_idx = -1
                    
                    # Find the target position by seeing which widget we're over
                    for i, widget in enumerate(self.item_widgets):
                        # Skip empty container widget if it exists
                        if hasattr(self, 'empty_container') and widget == self.empty_container:
                            continue
                            
                        # Check if widget has been deleted
                        if self.isDeleted(widget):
                            continue
                            
                        try:
                            # Get the widget geometry in the container's coordinate system
                            widget_rect = widget.geometry()
                            
                            # If drop is more than halfway down a widget, insert after it
                            if widget_rect.contains(pos):
                                mid_y = widget_rect.y() + widget_rect.height() / 2
                                if pos.y() < mid_y:
                                    target_idx = i  # Insert before this item
                                else:
                                    target_idx = i + 1  # Insert after this item
                                break
                        except RuntimeError:
                            print(f"Skipping deleted widget at index {i}")
                            continue
                    
                    # If we didn't find a specific widget or we're after the last one, put at the end
                    if target_idx == -1:
                        target_idx = len(self.item_widgets) - 1  # End of list
                        if hasattr(self, 'empty_container') and self.empty_container.isVisible():
                            target_idx = 0  # If empty state is showing, place at beginning
                    
                    # Don't do anything if dropping in the same place
                    if source_idx == target_idx or source_idx + 1 == target_idx:
                        event.accept()
                        return True
                    
                    # Reorder the favorites in the store
                    self.store.reorder_favorites(source_idx, target_idx)
                    
                    event.accept()
                    return True
                else:
                    event.ignore()
        
        return super().eventFilter(obj, event)
    
    def update_favorites_list(self):
        """Update the list of favorite commands"""
        try:
            print("Updating favorites list")
            
            # First, close any edit operations that might be in progress
            self.setFocus()
            
            # Create a brand new container widget to avoid any widget deletion issues
            # Use safe references to old widgets
            old_container = None
            old_scroll_area = None
            
            if hasattr(self, 'container') and not self.isDeleted(self.container):
                old_container = self.container
                
            if hasattr(self, 'scroll_area') and not self.isDeleted(self.scroll_area):
                old_scroll_area = self.scroll_area
            
            # Create a new scroll area
            new_scroll_area = QScrollArea()
            new_scroll_area.setWidgetResizable(True)
            new_scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
            new_scroll_area.setFrameShape(QFrame.Shape.NoFrame)
            new_scroll_area.setStyleSheet("""
                QScrollArea {
                    border: none;
                    background-color: transparent;
                }
                QScrollBar:vertical {
                    border: none;
                    background: #f0f0f0;
                    width: 8px;
                    border-radius: 4px;
                    margin: 0px;
                }
                QScrollBar::handle:vertical {
                    background: #bbbbbb;
                    min-height: 20px;
                    border-radius: 4px;
                }
                QScrollBar::handle:vertical:hover {
                    background: #999999;
                }
                QScrollBar::add-line:vertical, 
                QScrollBar::sub-line:vertical {
                    height: 0px;
                }
                QScrollBar::add-page:vertical, 
                QScrollBar::sub-page:vertical {
                    background: none;
                }
            """)
            
            # Create a new container with improved styling
            new_container = QWidget()
            new_container.setObjectName("favoritesContainer")
            new_container.setStyleSheet("""
                #favoritesContainer {
                    background-color: white;
                    border-radius: 8px;
                }
            """)
            new_container.setAcceptDrops(True)  # Enable drops
            
            # Create a new layout for the container
            new_layout = QVBoxLayout(new_container)
            new_layout.setContentsMargins(15, 15, 15, 15)
            new_layout.setSpacing(12)
            new_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
            
            # Reset our item widgets list
            self.item_widgets = []
            
            # Add favorite command widgets to the new container
            if self.store.favorite_commands:
                # First, remove the empty container if it exists
                if hasattr(self, 'empty_container'):
                    try:
                        # Check if the widget hasn't been deleted
                        if not self.isDeleted(self.empty_container):
                            self.empty_container.setVisible(False)
                    except (RuntimeError, Exception) as e:
                        # Widget already deleted or other error
                        print(f"Empty container handling: {e}")
                        pass
                
                # Add items to the new layout
                print(f"Adding {len(self.store.favorite_commands)} favorites to new layout")
                for i, favorite in enumerate(self.store.favorite_commands):
                    item = FavoriteCommandItem(favorite, i, self)
                    new_layout.addWidget(item)
                    self.item_widgets.append(item)
            else:
                # Create or show empty state
                empty_container = QWidget()
                empty_layout = QVBoxLayout(empty_container)
                empty_layout.setContentsMargins(20, 40, 20, 40)
                empty_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
                
                empty_icon = QLabel("❤️")
                empty_icon.setStyleSheet("""
                    font-size: 48px;
                    margin-bottom: 15px;
                """)
                empty_icon.setAlignment(Qt.AlignmentFlag.AlignCenter)
                empty_layout.addWidget(empty_icon)
                
                empty_label = QLabel("No favorite commands yet")
                empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
                empty_label.setStyleSheet("""
                    font-size: 18px;
                    color: #888;
                    margin-bottom: 10px;
                """)
                empty_layout.addWidget(empty_label)
                
                empty_hint = QLabel("Add favorites from the main window by clicking the heart icon")
                empty_hint.setAlignment(Qt.AlignmentFlag.AlignCenter)
                empty_hint.setStyleSheet("""
                    font-size: 14px;
                    color: #aaa;
                    margin-bottom: 20px;
                """)
                empty_layout.addWidget(empty_hint)
                
                new_layout.addWidget(empty_container)
                self.empty_container = empty_container
                self.item_widgets = [empty_container]
            
            # Add stretch at the end
            new_layout.addStretch(1)
            
            # Install event filter on new container
            new_container.installEventFilter(self)
            
            # Set the new container as the scroll area widget
            new_scroll_area.setWidget(new_container)
            
            # Update references
            self.scroll_area = new_scroll_area
            self.container = new_container
            self.container_layout = new_layout
            
            # Replace old widgets in parent layout
            if old_scroll_area:
                try:
                    if not self.isDeleted(old_scroll_area):
                        layout = self.layout()
                        if layout:
                            # Find the old scroll area in the layout
                            for i in range(layout.count()):
                                item = layout.itemAt(i)
                                if item and item.widget() == old_scroll_area:
                                    # Replace it with the new one
                                    layout.replaceWidget(old_scroll_area, new_scroll_area)
                                    break
                            
                            # Show the new scroll area
                            new_scroll_area.setVisible(True)
                            
                            # Schedule old widgets for deletion after we're done
                            old_scroll_area.setVisible(False)
                            old_scroll_area.setParent(None)
                            old_scroll_area.deleteLater()
                        else:
                            print("Error: Could not find layout")
                except (RuntimeError, Exception) as e:
                    print(f"Error handling old scroll area: {e}")
                    # If old scroll area was deleted, just add the new one directly
                    layout = self.layout()
                    if layout:
                        layout.addWidget(new_scroll_area)
            else:
                # If no old scroll area, add the new one
                layout = QVBoxLayout(self)
                layout.setContentsMargins(0, 0, 0, 0)
                layout.setSpacing(0)
                layout.addWidget(new_scroll_area)
            
            # Force the UI to update
            QApplication.processEvents()
            print("Favorites list updated successfully")
            
        except Exception as e:
            print(f"Error updating favorites list: {e}")
            import traceback
            traceback.print_exc()