from PyQt6.QtCore import Qt, QSize, QPropertyAnimation, QEasingCurve
from PyQt6.QtGui import QIcon, QAction, QPalette, QColor, QFont
from PyQt6.QtWidgets import (
    QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, 
    QToolBar, QStatusBar, QApplication, QLabel,
    QGraphicsDropShadowEffect, QFrame
)

from models import PluginStore
from ui.banner_view import BannerView


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        
        # Set up the store
        self.store = PluginStore.get_instance()
        
        # Set up window properties
        self.setWindowTitle("NotePlan Plugin Launcher")
        self.setFixedHeight(80)  # Lock height to just fit one row of emojis
        self.setMinimumWidth(600)  # Allow width to be resized
        
        # Set up theme
        self.dark_mode = False
        self.setup_theme()
        
        # Set up central widget
        self.central_widget = QWidget()
        self.setCentralWidget(self.central_widget)
        
        # Create layout - no margins or spacing to maximize space
        self.main_layout = QVBoxLayout(self.central_widget)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.setSpacing(0)
        
        # Create main UI with banner directly at the top
        self.setup_ui()
        
        # Remove status bar for more compact UI
        self.setStatusBar(None)
        
        # Connect signals for loading
        self.store.loading_changed.connect(self.on_loading_changed)
        self.store.error_occurred.connect(self.on_error_occurred)
        
        # Scan plugins on startup
        self.store.scan_plugins()
    
    def setup_theme(self):
        """Set up application theme and styles"""
        # Set global application font
        font = QFont("SF Pro Display", 10)
        QApplication.setFont(font)
        
        # Apply stylesheet for light mode by default
        self.apply_theme()
    
    
    def setup_ui(self):
        """Set up the main UI with banner view"""
        # Create the banner view with theme toggle capability
        self.banner_view = BannerView(self)
        
        # Connect theme toggle signal from banner view
        self.banner_view.theme_toggle_requested.connect(self.toggle_theme)
        
        # Add to main layout - this will be the only widget in the window
        self.main_layout.addWidget(self.banner_view)
    
    def toggle_theme(self):
        """Toggle between light and dark mode"""
        try:
            self.dark_mode = not self.dark_mode
            self.apply_theme()
            
            # Update banner view with error handling
            if hasattr(self, 'banner_view') and self.banner_view is not None:
                try:
                    self.banner_view.set_dark_mode(self.dark_mode)
                except Exception as e:
                    print(f"Error updating banner view: {e}")
        except Exception as e:
            print(f"Error toggling theme: {e}")
    
    def apply_theme(self):
        """Apply the current theme to the application"""
        if self.dark_mode:
            self.setStyleSheet("""
                QMainWindow, QWidget {
                    background-color: #1e1e1e;
                    color: #f0f0f0;
                }
                QScrollArea, QScrollBar {
                    background-color: #1e1e1e;
                }
                QScrollBar:vertical {
                    border: none;
                    background-color: #2d2d2d;
                    width: 10px;
                    margin: 0px;
                }
                QScrollBar::handle:vertical {
                    background-color: #5a5a5a;
                    border-radius: 5px;
                    min-height: 20px;
                }
                QScrollBar::add-line:vertical, QScrollBar::sub-line:vertical {
                    height: 0px;
                }
                QStatusBar {
                    background-color: #2d2d2d;
                    color: #bbbbbb;
                }
                QFrame[frameShape="4"] {
                    background-color: #444444;
                }
            """)
        else:
            self.setStyleSheet("""
                QMainWindow, QWidget {
                    background-color: #f8f8f8;
                    color: #333333;
                }
                QScrollArea, QScrollBar {
                    background-color: #f8f8f8;
                }
                QScrollBar:vertical {
                    border: none;
                    background-color: #f0f0f0;
                    width: 10px;
                    margin: 0px;
                }
                QScrollBar::handle:vertical {
                    background-color: #cccccc;
                    border-radius: 5px;
                    min-height: 20px;
                }
                QScrollBar::add-line:vertical, QScrollBar::sub-line:vertical {
                    height: 0px;
                }
                QStatusBar {
                    background-color: #f0f0f0;
                    color: #666666;
                }
                QFrame[frameShape="4"] {
                    background-color: #dddddd;
                }
            """)
    
    
    def on_loading_changed(self, is_loading):
        """When loading state changes"""
        # We've removed the status bar, so we don't need to show messages
        pass
    
    def on_error_occurred(self, error):
        """When an error occurs"""
        # We've removed the status bar, so we don't need to show messages
        print(f"Error: {error}")
        
    def closeEvent(self, event):
        """Handle window close event to save plugin order"""
        try:
            # Save current plugin order before closing
            if hasattr(self, 'banner_view') and self.banner_view is not None:
                self.banner_view._save_plugin_order()
        except Exception as e:
            print(f"Error saving plugin order on close: {e}")
        
        # Accept the close event
        event.accept()