import subprocess
import time
from PyQt6.QtCore import Qt, pyqtSignal, QPropertyAnimation, QEasingCurve, QSize
from PyQt6.QtGui import QIcon, QColor, QPalette, QFont
from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QScrollArea, QLabel, 
    QPushButton, QFrame, QHBoxLayout, QSizePolicy,
    QProgressBar, QGraphicsDropShadowEffect, QApplication
)

from models import PluginStore, Plugin, PluginCommand


class CommandWidget(QWidget):
    """Widget representing a single plugin command"""
    
    def __init__(self, plugin, command, parent=None):
        super().__init__(parent)
        self.plugin = plugin
        self.command = command
        self.store = PluginStore.get_instance()
        self.is_hovered = False
        
        # Create layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(15, 10, 15, 10)
        layout.setSpacing(8)  # Reduce spacing to fit more content
        
        # Command info
        info_layout = QVBoxLayout()
        info_layout.setSpacing(6)
        
        self.name_label = QLabel(f"<b>{command.name}</b>")
        self.name_label.setStyleSheet("font-size: 14px;")
        # Show full name in tooltip and set long duration
        self.name_label.setToolTip(command.name)
        try:
            self.name_label.setToolTipDuration(5000)  # 5 seconds duration
        except:
            pass
        self.name_label.setMinimumWidth(100)  # Minimum width
        self.name_label.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)  # Allow expanding
        self.name_label.setWordWrap(True)  # Enable word wrap for long names
        info_layout.addWidget(self.name_label)
        
        if command.description:
            self.desc_label = QLabel(command.description)
            self.desc_label.setWordWrap(True)
            self.desc_label.setStyleSheet("color: #666; font-size: 13px;")
            self.desc_label.setMinimumWidth(100)  # Minimum width
            self.desc_label.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)  # Allow expanding
            info_layout.addWidget(self.desc_label)
        
        layout.addLayout(info_layout, 1)
        
        # Button container with right alignment
        button_container = QWidget()
        button_layout = QHBoxLayout(button_container)
        button_layout.setContentsMargins(0, 0, 0, 0)
        button_layout.setSpacing(8)
        
        # Favorite button with better styling
        self.fav_button = QPushButton()
        self.fav_button.setFixedSize(36, 36)
        self.fav_button.setFlat(True)
        self.fav_button.setCursor(Qt.CursorShape.PointingHandCursor)
        
        # Connect directly to store's toggle_favorite method
        self.fav_button.clicked.connect(lambda: self.toggle_favorite())
        button_layout.addWidget(self.fav_button)
        
        # Run button with better styling
        self.run_button = QPushButton("Run")
        self.run_button.setFixedSize(60, 36)  # Make button narrower
        self.run_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.run_button.setFont(QFont("SF Pro Display", 12))
        
        # Connect with explicit lambda to make debugging easier
        self.run_button.clicked.connect(lambda checked=False: self.run_command())
        
        # Make sure button is clearly clickable
        self.run_button.setStyleSheet("""
            QPushButton {
                background-color: #4a90e2;
                color: white;
                border-radius: 4px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #2a70c2;
            }
        """)
        button_layout.addWidget(self.run_button)
        
        layout.addWidget(button_container)
        
        # Set widget properties
        min_height = layout.sizeHint().height() + 10
        self.setMinimumHeight(min_height)
        self.setAutoFillBackground(True)
        
        # Add shadow effect
        self.shadow_effect = QGraphicsDropShadowEffect()
        self.shadow_effect.setBlurRadius(10)
        self.shadow_effect.setColor(QColor(0, 0, 0, 50))
        self.shadow_effect.setOffset(0, 2)
        self.setGraphicsEffect(self.shadow_effect)
        
        # Default background color
        self.default_style = "background-color: #f8f8f8; border-radius: 8px; margin: 3px;"
        self.hover_style = "background-color: #f0f0f0; border-radius: 8px; margin: 3px;"
        self.setStyleSheet(self.default_style)
        
        # Update favorite status
        self.update_favorite_status()
        self.store.favorites_updated.connect(self.update_favorite_status)
        
        # Add hover animations
        self.installEventFilter(self)
    
    def eventFilter(self, obj, event):
        """Handle hover events for custom styling"""
        if event.type() == event.Type.Enter:
            self.is_hovered = True
            self.update_style()
            return True
        elif event.type() == event.Type.Leave:
            self.is_hovered = False
            self.update_style()
            return True
        return super().eventFilter(obj, event)
    
    def update_style(self):
        """Update widget style based on hover state"""
        # Update shadow
        self.shadow_effect.setBlurRadius(20 if self.is_hovered else 10)
        self.shadow_effect.setOffset(0, 4 if self.is_hovered else 2)
        
        # Update background
        self.setStyleSheet(self.hover_style if self.is_hovered else self.default_style)
        
        # Animate run button if hovered
        if self.is_hovered:
            animation = QPropertyAnimation(self.run_button, b"minimumWidth")
            animation.setDuration(200)
            animation.setStartValue(60)  # Updated to match the fixed width we set earlier
            animation.setEndValue(70)
            animation.setEasingCurve(QEasingCurve.Type.OutCubic)
            animation.start()
    
    def toggle_favorite(self):
        """Toggle favorite status"""
        try:
            # Ensure necessary imports are available
            import subprocess
            import time
            
            # Show wait cursor
            QApplication.instance().setOverrideCursor(Qt.CursorShape.WaitCursor)
            
            # Toggle favorite status
            self.store.toggle_favorite(self.plugin, self.command)
            self.update_favorite_status()
            
            # Manually trigger favorites view refresh
            # Find the main window to refresh favorites view
            main_window = None
            parent = self
            while parent is not None:
                parent = parent.parent()
                if hasattr(parent, 'banner_view'):
                    main_window = parent
                    break
            
            if main_window and hasattr(main_window, 'banner_view'):
                # Force banner view to update
                main_window.banner_view.update_plugins()
            
            # Restore cursor
            QApplication.instance().restoreOverrideCursor()
        except Exception as e:
            # Make sure cursor is restored
            try:
                QApplication.instance().restoreOverrideCursor()
            except:
                pass
            print(f"Error toggling favorite: {e}")
    
    def update_favorite_status(self):
        """Update favorite button appearance"""
        is_favorite = self.store.is_favorite(self.plugin, self.command)
        if is_favorite:
            self.fav_button.setText("★")
            self.fav_button.setStyleSheet("""
                color: gold;
                font-size: 20px;
                background-color: transparent;
                border: none;
            """)
            self.fav_button.setToolTip("Remove from favorites")
        else:
            self.fav_button.setText("☆")
            self.fav_button.setStyleSheet("""
                color: #bbbbbb;
                font-size: 20px;
                background-color: transparent;
                border: none;
            """)
            self.fav_button.setToolTip("Add to favorites")
    
    def run_command(self):
        """Run the command"""
        try:
            # Ensure these are imported directly in the method
            import subprocess
            import time
            
            # Show wait cursor
            QApplication.instance().setOverrideCursor(Qt.CursorShape.WaitCursor)
            
            # Try to execute via store first
            try:
                self.store.execute_command(self.plugin, self.command)
            except Exception as store_error:
                print(f"Error executing through store: {store_error}")
                
                # Direct fallback execution
                plugin_id = self.plugin.id.replace(" ", "%20")
                command_name = self.command.name.replace(" ", "%20")
                url = f"noteplan://x-callback-url/runPlugin?pluginID={plugin_id}&command={command_name}"
                
                print(f"Trying direct execution of: {url}")
                
                # First try to open NotePlan
                subprocess.run(["open", "-a", "NotePlan"], check=False)
                
                # Wait a moment for NotePlan to be ready
                time.sleep(0.3)
                
                # Then open the URL
                subprocess.run(["open", url], check=False)
            
            # Close the menu if the command was run from a plugin widget
            # First find the parent plugin widget
            parent_widget = self.parent()
            while parent_widget and not isinstance(parent_widget, PluginWidget):
                parent_widget = parent_widget.parent()
                
            # If we found a plugin widget parent, close its menu
            if parent_widget and hasattr(parent_widget, 'toggle_expansion'):
                if parent_widget.expanded:
                    parent_widget.toggle_expansion(None)
            
            # Restore cursor
            QApplication.instance().restoreOverrideCursor()
                
        except Exception as e:
            # Make sure cursor is restored
            try:
                QApplication.instance().restoreOverrideCursor()
            except:
                pass
            print(f"Error executing command: {e}")


class PluginWidget(QWidget):
    """Widget representing a plugin with its commands"""
    
    def __init__(self, plugin, parent=None):
        super().__init__(parent)
        self.plugin = plugin
        self.expanded = False
        self.animation = None
        
        # Create layout
        self.main_layout = QVBoxLayout(self)
        self.main_layout.setContentsMargins(0, 0, 0, 8)
        self.main_layout.setSpacing(8)
        
        # Header frame with gradient and shadow
        self.header_frame = QFrame()
        self.header_frame.setCursor(Qt.CursorShape.PointingHandCursor)
        self.header_frame.setStyleSheet("""
            QFrame {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0, 
                                            stop:0 #6a8caf, stop:1 #99bdda);
                border-radius: 8px;
                min-height: 50px;
            }
        """)
        
        # Add shadow to header
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(15)
        shadow.setColor(QColor(0, 0, 0, 60))
        shadow.setOffset(0, 3)
        self.header_frame.setGraphicsEffect(shadow)
        
        header_layout = QHBoxLayout(self.header_frame)
        header_layout.setContentsMargins(15, 10, 15, 10)
        
        # Plugin info with better styling
        self.expand_label = QLabel("▶")
        self.expand_label.setStyleSheet("""
            font-size: 14px;
            color: white;
            padding-right: 5px;
        """)
        header_layout.addWidget(self.expand_label)
        
        self.name_label = QLabel(f"<b>{plugin.name}</b>")
        self.name_label.setStyleSheet("""
            font-size: 16px;
            color: white;
        """)
        header_layout.addWidget(self.name_label)
        
        self.cmd_count = QLabel(f"({len(plugin.commands)} commands)")
        self.cmd_count.setStyleSheet("""
            color: rgba(255, 255, 255, 0.8);
            margin-left: 10px;
        """)
        header_layout.addWidget(self.cmd_count)
        
        # Add version info
        version_label = QLabel(f"v{plugin.version}")
        version_label.setStyleSheet("""
            color: rgba(255, 255, 255, 0.7);
            font-size: 12px;
        """)
        
        header_layout.addStretch(1)
        header_layout.addWidget(version_label)
        
        self.main_layout.addWidget(self.header_frame)
        
        # Commands container with animation support
        self.commands_container = QWidget()
        commands_container_layout = QVBoxLayout(self.commands_container)
        commands_container_layout.setContentsMargins(0, 0, 0, 0)
        commands_container_layout.setSpacing(0)
        
        self.commands_widget = QWidget()
        self.commands_layout = QVBoxLayout(self.commands_widget)
        self.commands_layout.setContentsMargins(20, 8, 20, 8)  # Add right margin to ensure text doesn't get too close to the edge
        self.commands_layout.setSpacing(8)
        
        # Create command widgets
        for command in plugin.commands:
            cmd_widget = CommandWidget(plugin, command)
            self.commands_layout.addWidget(cmd_widget)
        
        commands_container_layout.addWidget(self.commands_widget)
        self.commands_container.setMaximumHeight(0)
        self.main_layout.addWidget(self.commands_container)
        
        # Connect signals
        self.header_frame.mousePressEvent = self.toggle_expansion
        
        # Add hover effect for header
        self.header_frame.enterEvent = self.on_header_hover_enter
        self.header_frame.leaveEvent = self.on_header_hover_leave
    
    def on_header_hover_enter(self, event):
        """Handle mouse hover enter event on header"""
        self.header_frame.setStyleSheet("""
            QFrame {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0, 
                                          stop:0 #7a9cbf, stop:1 #a9cdea);
                border-radius: 8px;
                min-height: 50px;
            }
        """)
    
    def on_header_hover_leave(self, event):
        """Handle mouse hover leave event on header"""
        self.header_frame.setStyleSheet("""
            QFrame {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0, 
                                          stop:0 #6a8caf, stop:1 #99bdda);
                border-radius: 8px;
                min-height: 50px;
            }
        """)
    
    def toggle_expansion(self, event):
        """Toggle expansion of commands list with animation"""
        self.expanded = not self.expanded
        self.expand_label.setText("▼" if self.expanded else "▶")
        
        # Calculate the full height of commands
        full_height = self.commands_layout.sizeHint().height() + 16  # Add padding
        
        # Create animation
        self.animation = QPropertyAnimation(self.commands_container, b"maximumHeight")
        self.animation.setDuration(300)
        self.animation.setEasingCurve(QEasingCurve.Type.OutCubic)
        
        if self.expanded:
            # Expand
            self.animation.setStartValue(0)
            self.animation.setEndValue(full_height)
        else:
            # Collapse
            self.animation.setStartValue(full_height)
            self.animation.setEndValue(0)
        
        self.animation.start()


class PluginListView(QWidget):
    """Main view displaying the list of plugins"""
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.store = PluginStore.get_instance()
        self.search_text = ""
        
        # Create layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        
        # Loading bar with better styling
        self.loading_container = QWidget()
        loading_layout = QVBoxLayout(self.loading_container)
        loading_layout.setContentsMargins(0, 5, 0, 5)
        
        self.loading_bar = QProgressBar()
        self.loading_bar.setTextVisible(False)
        self.loading_bar.setRange(0, 0)  # Indeterminate progress
        self.loading_bar.setMaximumHeight(4)
        self.loading_bar.setStyleSheet("""
            QProgressBar {
                background-color: transparent;
                border: none;
                border-radius: 2px;
            }
            QProgressBar::chunk {
                background-color: #4a90e2;
                border-radius: 2px;
            }
        """)
        loading_layout.addWidget(self.loading_bar)
        
        self.loading_container.setVisible(False)
        layout.addWidget(self.loading_container)
        
        # Scroll area for plugins with custom styling
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        
        # Add smooth scroll animation
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.scroll_area.verticalScrollBar().setSingleStep(10)  # Smoother scrolling
        
        # Container for plugins with spacing
        self.container = QWidget()
        self.container_layout = QVBoxLayout(self.container)
        self.container_layout.setContentsMargins(15, 15, 15, 15)
        self.container_layout.setSpacing(15)
        self.container_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        
        # Empty state label with icon
        self.empty_widget = QWidget()
        empty_layout = QVBoxLayout(self.empty_widget)
        empty_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        empty_icon = QLabel("📦")
        empty_icon.setStyleSheet("font-size: 48px; margin-bottom: 10px;")
        empty_icon.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        self.empty_label = QLabel("No plugins found")
        self.empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.empty_label.setStyleSheet("font-size: 20px; color: #666; margin: 10px;")
        
        empty_layout.addWidget(empty_icon)
        empty_layout.addWidget(self.empty_label)
        empty_layout.addStretch(1)
        
        self.container_layout.addWidget(self.empty_widget)
        self.empty_widget.setVisible(False)
        
        # Error state with icon
        self.error_widget = QWidget()
        error_layout = QVBoxLayout(self.error_widget)
        error_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        error_icon = QLabel("⚠️")
        error_icon.setStyleSheet("font-size: 48px; margin-bottom: 10px;")
        error_icon.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        self.error_label = QLabel()
        self.error_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.error_label.setStyleSheet("font-size: 18px; color: #c00; margin: 10px;")
        self.error_label.setWordWrap(True)
        
        error_layout.addWidget(error_icon)
        error_layout.addWidget(self.error_label)
        error_layout.addStretch(1)
        
        self.container_layout.addWidget(self.error_widget)
        self.error_widget.setVisible(False)
        
        self.scroll_area.setWidget(self.container)
        layout.addWidget(self.scroll_area)
        
        # Connect signals
        self.store.plugins_updated.connect(self.update_plugin_list)
        self.store.loading_changed.connect(self.on_loading_changed)
        self.store.error_occurred.connect(self.on_error_occurred)
        
        # Initialize
        self.update_plugin_list()
    
    def set_search_text(self, text):
        """Set search text and update view"""
        self.search_text = text.lower()
        self.update_plugin_list()
    
    def on_loading_changed(self, is_loading):
        """When loading state changes"""
        self.loading_container.setVisible(is_loading)
        
        # Add fade animation for loading bar
        if is_loading:
            # Reset the container in case it was previously hidden
            self.loading_container.setStyleSheet("opacity: 0;")
            
            # Animate fade in
            self._animate_opacity(self.loading_container, 0, 1, 300)
    
    def on_error_occurred(self, error):
        """When an error occurs"""
        self.error_label.setText(error)
        self.error_widget.setVisible(True)
        
        # Animate the error appearance
        self._animate_opacity(self.error_widget, 0, 1, 300)
    
    def _animate_opacity(self, widget, start_value, end_value, duration):
        """Animate widget opacity"""
        # Use stylesheet-based opacity animation
        widget.setStyleSheet(f"opacity: {start_value};")
        widget.setVisible(True)
        
        # Create animation
        animation = QPropertyAnimation(widget, b"windowOpacity")
        animation.setDuration(duration)
        animation.setStartValue(start_value)
        animation.setEndValue(end_value)
        animation.setEasingCurve(QEasingCurve.Type.OutCubic)
        animation.start()
        
        # Update stylesheet at end of animation
        if end_value == 1:
            widget.setStyleSheet("opacity: 1;")
    
    def update_plugin_list(self):
        """Update the plugin list"""
        # Clear existing widgets (except for labels)
        for i in reversed(range(self.container_layout.count())):
            item = self.container_layout.itemAt(i)
            if item.widget() != self.empty_widget and item.widget() != self.error_widget:
                widget = item.widget()
                self.container_layout.removeItem(item)
                if widget:
                    widget.deleteLater()
        
        # Hide state widgets
        self.empty_widget.setVisible(False)
        self.error_widget.setVisible(False)
        
        # Filter plugins with improved search
        filtered_plugins = []
        if self.search_text:
            search_terms = self.search_text.lower().split()
            for plugin in self.store.plugins:
                # Check if plugin name matches all search terms
                if all(term in plugin.name.lower() for term in search_terms):
                    filtered_plugins.append(plugin)
                    continue
                
                # Check if any command matches all search terms
                for cmd in plugin.commands:
                    if all(term in cmd.name.lower() or term in cmd.description.lower() 
                           for term in search_terms):
                        filtered_plugins.append(plugin)
                        break
        else:
            filtered_plugins = self.store.plugins
        
        # Add plugin widgets with staggered animations
        if filtered_plugins:
            for i, plugin in enumerate(filtered_plugins):
                plugin_widget = PluginWidget(plugin)
                plugin_widget.setStyleSheet("opacity: 0;")
                self.container_layout.addWidget(plugin_widget)
                
                # Create staggered fade-in animation
                delay = i * 50  # Stagger by 50ms per item
                animation = QPropertyAnimation(plugin_widget, b"windowOpacity")
                animation.setDuration(300)
                animation.setStartValue(0)
                animation.setEndValue(1)
                animation.setEasingCurve(QEasingCurve.Type.OutCubic)
                animation.setLoopCount(1)
                
                # Start after delay
                animation.start(QPropertyAnimation.DeletionPolicy.KeepWhenStopped)
                
                # Reset style at end of animation
                # This is a hacky way to wait for the animation to finish
                QApplication.instance().processEvents()
                plugin_widget.setStyleSheet("")
        else:
            if self.store.error:
                self.error_widget.setVisible(True)
                self._animate_opacity(self.error_widget, 0, 1, 300)
            elif self.search_text:
                self.empty_label.setText(f"No plugins match '{self.search_text}'")
                self.empty_widget.setVisible(True)
                self._animate_opacity(self.empty_widget, 0, 1, 300)
            elif not self.store.plugins:
                self.empty_label.setText("No plugins found")
                self.empty_widget.setVisible(True)
                self._animate_opacity(self.empty_widget, 0, 1, 300)
        
        # Add stretch to push widgets to top
        self.container_layout.addStretch(1)