import subprocess
import time
from PyQt6.QtCore import (
    Qt, QTimer, QObject, QEvent
)
from PyQt6.QtGui import QColor
from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QScrollArea, QLabel, 
    QPushButton, QFrame, QHBoxLayout, QApplication,
    QToolButton, QMainWindow, QCheckBox,
    QGraphicsDropShadowEffect
)

from models import PluginStore


# Global reference to keep manager window alive
_manager_window_ref = None

def _clear_global_reference():
    """Helper function to clear the global window reference safely"""
    global _manager_window_ref
    print("Clearing global window reference")
    _manager_window_ref = None

class WindowCloseFilter(QObject):
    """Event filter to handle window closing and cleanup"""
    
    def eventFilter(self, obj, event):
        """Filter window close events"""
        if event.type() == QEvent.Type.Close:
            # Print message on close
            print("Plugins visibility manager window closing...")
            
            # Allow the event to propagate
            return False
        
        # Let other events pass through
        return super().eventFilter(obj, event)


class PluginVisibilityItem(QWidget):
    """Widget representing a plugin with a show/hide checkbox"""
    
    def __init__(self, plugin, parent=None):
        super().__init__(parent)
        self.plugin = plugin
        self.store = PluginStore.get_instance()
        self.parent_view = parent
        
        # Create overall frame with shadow effect
        self.setObjectName("pluginItem")
        
        # Add a subtle shadow effect
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(8)
        shadow.setColor(QColor(0, 0, 0, 40))
        shadow.setOffset(0, 2)
        self.setGraphicsEffect(shadow)
        
        # Create layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(12, 12, 12, 12)
        layout.setSpacing(12)
        
        # Checkbox for show/hide
        self.visibility_checkbox = QCheckBox()
        self.visibility_checkbox.setText("")
        self.visibility_checkbox.setToolTip("Show/Hide plugin in banner")
        self.visibility_checkbox.setCursor(Qt.CursorShape.PointingHandCursor)
        self.visibility_checkbox.setChecked(not self.store.is_plugin_hidden(plugin.id))
        self.visibility_checkbox.stateChanged.connect(self.toggle_visibility)
        layout.addWidget(self.visibility_checkbox)
        
        # Plugin info container
        info_container = QWidget()
        info_container.setStyleSheet("background: transparent;")
        info_layout = QVBoxLayout(info_container)
        info_layout.setContentsMargins(0, 0, 0, 0)
        info_layout.setSpacing(4)
        
        # Plugin name (don't repeat the emoji)
        plugin_name_label = QLabel(f"{plugin.name}")
        plugin_name_label.setStyleSheet("""
            font-size: 16px;
            font-weight: bold;
            color: #333;
        """)
        info_layout.addWidget(plugin_name_label)
        
        # Plugin description
        if plugin.description:
            desc_label = QLabel(plugin.description[:100] + ("..." if len(plugin.description) > 100 else ""))
            desc_label.setStyleSheet("""
                font-size: 12px;
                color: #666;
            """)
            desc_label.setWordWrap(True)
            info_layout.addWidget(desc_label)
        
        # Command count
        cmd_count_label = QLabel(f"{len(plugin.commands)} command{'s' if len(plugin.commands) != 1 else ''}")
        cmd_count_label.setStyleSheet("""
            font-size: 12px;
            color: #888;
        """)
        info_layout.addWidget(cmd_count_label)
        
        layout.addWidget(info_container, 1)
        
        # Set widget properties
        self.setFixedHeight(layout.sizeHint().height() + 10)
        self.setAutoFillBackground(True)
        
        # Default background color and border styling
        self.update_style()
    
    def update_style(self):
        """Update styling based on visibility state"""
        if self.visibility_checkbox.isChecked():
            self.setStyleSheet("""
                #pluginItem {
                    background-color: white;
                    border: 1px solid #e0e0e0;
                    border-radius: 8px;
                }
                #pluginItem:hover {
                    background-color: #f8f8f8;
                    border: 1px solid #d0d0d0;
                }
            """)
        else:
            self.setStyleSheet("""
                #pluginItem {
                    background-color: #f5f5f5;
                    border: 1px solid #e0e0e0;
                    border-radius: 8px;
                    opacity: 0.8;
                }
                #pluginItem:hover {
                    background-color: #f0f0f0;
                    border: 1px solid #d0d0d0;
                }
            """)
    
    def toggle_visibility(self):
        """Toggle plugin visibility and update UI"""
        try:
            # Toggle visibility in the store
            self.store.toggle_plugin_visibility(self.plugin.id)
            
            # Update the widget style
            self.update_style()
            
            # Add a small delay so user can see the checkbox change
            QTimer.singleShot(50, lambda: QApplication.processEvents())
        except Exception as e:
            print(f"Error toggling plugin visibility: {e}")


class PluginsVisibilityManagerWindow(QMainWindow):
    """Separate window for managing plugin visibility in the banner"""
    
    def __init__(self, parent=None):
        global _manager_window_ref
        
        super().__init__(None)  # Create as a separate independent window
        self.setWindowTitle("Show/Hide Plugins")
        self.setMinimumSize(600, 450)
        
        # Set up window flags to ensure it behaves properly
        self.setWindowFlags(
            Qt.WindowType.Window |
            Qt.WindowType.WindowTitleHint |
            Qt.WindowType.WindowCloseButtonHint |
            Qt.WindowType.WindowStaysOnTopHint  # Make it stay on top
        )
        
        # Install event filter for window close
        self.event_filter = WindowCloseFilter()
        self.installEventFilter(self.event_filter)
        
        # Keep a global reference to prevent garbage collection
        _manager_window_ref = self
        
        self.store = PluginStore.get_instance()
        self.parent_widget = parent
        
        # Create central widget with background color
        central_widget = QWidget()
        central_widget.setStyleSheet("background-color: #f9f9f9;")
        self.setCentralWidget(central_widget)
        
        # Main layout
        layout = QVBoxLayout(central_widget)
        layout.setContentsMargins(20, 20, 20, 20)
        layout.setSpacing(15)
        
        # Header with title and instructions
        header_widget = QWidget()
        header_widget.setStyleSheet("background-color: #4a90e2; border-radius: 8px;")
        header_layout = QVBoxLayout(header_widget)
        header_layout.setContentsMargins(15, 15, 15, 15)
        
        title = QLabel("Show/Hide Plugins")
        title.setStyleSheet("font-size: 18px; font-weight: bold; color: white;")
        title.setAlignment(Qt.AlignmentFlag.AlignCenter)
        header_layout.addWidget(title)
        
        instructions = QLabel("Check to show a plugin in the banner · Uncheck to hide it")
        instructions.setStyleSheet("font-size: 13px; color: rgba(255, 255, 255, 0.9); margin-top: 5px;")
        instructions.setAlignment(Qt.AlignmentFlag.AlignCenter)
        header_layout.addWidget(instructions)
        
        layout.addWidget(header_widget)
        
        # Add the plugins list with a border and shadow
        plugins_container = QFrame()
        plugins_container.setFrameShape(QFrame.Shape.StyledPanel)
        plugins_container.setStyleSheet("""
            QFrame {
                background-color: white;
                border: 1px solid #ddd;
                border-radius: 8px;
            }
        """)
        
        # Add drop shadow effect
        shadow = QGraphicsDropShadowEffect()
        shadow.setBlurRadius(15)
        shadow.setColor(QColor(0, 0, 0, 30))
        shadow.setOffset(0, 2)
        plugins_container.setGraphicsEffect(shadow)
        
        plugins_layout = QVBoxLayout(plugins_container)
        plugins_layout.setContentsMargins(2, 2, 2, 2)
        
        self.plugins_view = PluginsVisibilityView(self)
        plugins_layout.addWidget(self.plugins_view)
        
        layout.addWidget(plugins_container, 1)
        
        # Bottom buttons with improved styling
        buttons_container = QWidget()
        buttons_container.setStyleSheet("background-color: transparent;")
        buttons_layout = QHBoxLayout(buttons_container)
        buttons_layout.setContentsMargins(0, 5, 0, 0)
        
        buttons_layout.addStretch(1)
        
        close_button = QPushButton("Close")
        close_button.setCursor(Qt.CursorShape.PointingHandCursor)
        close_button.clicked.connect(self.delayed_close)
        close_button.setMinimumWidth(100)
        close_button.setStyleSheet("""
            QPushButton {
                background-color: #4a90e2;
                color: white;
                border: none;
                border-radius: 6px;
                padding: 10px 20px;
                font-weight: bold;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: #3a80d2;
            }
            QPushButton:pressed {
                background-color: #2a70c2;
            }
        """)
        buttons_layout.addWidget(close_button)
        
        layout.addWidget(buttons_container)
    
    def delayed_close(self):
        """Close the window after a small delay"""
        print("Close button clicked, will close window after delay...")
        # Schedule window close after a short delay
        QTimer.singleShot(200, self.close)
    
    def closeEvent(self, event):
        """Handle window close event"""
        global _manager_window_ref
        
        print("Plugins manager window closeEvent called")
        
        # Force a refresh of the banner in the main window
        self.store = PluginStore.get_instance()
        try:
            print("Emitting plugins_updated signal on window close")
            self.store.plugins_updated.emit()
        except Exception as e:
            print(f"Error updating plugins: {e}")
        
        # Accept the close event
        event.accept()
        
        # Clear the global reference after a delay
        # This prevents the window from being garbage collected too soon
        QTimer.singleShot(500, lambda: _clear_global_reference())


class PluginsVisibilityView(QWidget):
    """View displaying plugin visibility toggles"""
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.store = PluginStore.get_instance()
        self.item_widgets = []  # Keep track of item widgets
        
        # Create layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        
        # Improved scroll area with nicer styling
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        self.scroll_area.setStyleSheet("""
            QScrollArea {
                border: none;
                background-color: transparent;
            }
            QScrollBar:vertical {
                border: none;
                background: #f0f0f0;
                width: 8px;
                border-radius: 4px;
                margin: 0px;
            }
            QScrollBar::handle:vertical {
                background: #bbbbbb;
                min-height: 20px;
                border-radius: 4px;
            }
            QScrollBar::handle:vertical:hover {
                background: #999999;
            }
            QScrollBar::add-line:vertical, 
            QScrollBar::sub-line:vertical {
                height: 0px;
            }
            QScrollBar::add-page:vertical, 
            QScrollBar::sub-page:vertical {
                background: none;
            }
        """)
        
        # Container with improved styling
        self.container = QWidget()
        self.container.setObjectName("pluginsContainer")
        self.container.setStyleSheet("""
            #pluginsContainer {
                background-color: white;
                border-radius: 8px;
            }
        """)
        
        self.container_layout = QVBoxLayout(self.container)
        self.container_layout.setContentsMargins(15, 15, 15, 15)
        self.container_layout.setSpacing(12)
        self.container_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        
        # Add plugin items
        self.update_plugins_list()
        
        self.scroll_area.setWidget(self.container)
        layout.addWidget(self.scroll_area)
        
        # Connect signals
        self.store.plugins_updated.connect(self.update_plugins_list)
    
    def update_plugins_list(self):
        """Update the list of plugins"""
        try:
            print("Updating plugins visibility list")
            
            # Clear existing items
            for widget in self.item_widgets:
                self.container_layout.removeWidget(widget)
                if widget is not None:
                    widget.deleteLater()
            
            self.item_widgets = []
            
            # No header with show all/hide all buttons anymore
            
            # Add plugin visibility items
            if self.store.plugins:
                for plugin in self.store.plugins:
                    item = PluginVisibilityItem(plugin, self)
                    self.container_layout.addWidget(item)
                    self.item_widgets.append(item)
            else:
                # Empty state
                empty_label = QLabel("No plugins found")
                empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
                empty_label.setStyleSheet("color: #888888; padding: 20px;")
                self.container_layout.addWidget(empty_label)
                self.item_widgets.append(empty_label)
            
            # Add stretch at the end
            self.container_layout.addStretch(1)
            
            # Force the UI to update
            QApplication.processEvents()
            print("Plugins visibility list updated successfully")
            
        except Exception as e:
            print(f"Error updating plugins list: {e}")
            import traceback
            traceback.print_exc()
    
    # Show All and Hide All methods removed as requested