from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QScrollArea, QLabel, 
    QPushButton, QFrame, QHBoxLayout
)

from models import PluginStore
from utils import relative_time


class RecentCommandItem(QWidget):
    """Widget representing a recent command"""
    
    def __init__(self, recent, parent=None):
        super().__init__(parent)
        self.recent = recent
        self.store = PluginStore.get_instance()
        
        # Create layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(10, 5, 10, 5)
        
        # Command info
        info_layout = QVBoxLayout()
        self.name_label = QLabel(f"<b>{recent.command_name}</b>")
        info_layout.addWidget(self.name_label)
        
        self.plugin_label = QLabel(recent.plugin_name)
        self.plugin_label.setStyleSheet("color: #666;")
        info_layout.addWidget(self.plugin_label)
        
        layout.addLayout(info_layout, 1)
        
        # Timestamp
        self.time_label = QLabel(relative_time(recent.timestamp))
        self.time_label.setStyleSheet("color: #666;")
        layout.addWidget(self.time_label)
        
        # Run button
        self.run_button = QPushButton("Run")
        self.run_button.setFixedSize(60, 32)
        self.run_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.run_button.clicked.connect(self.run_command)
        layout.addWidget(self.run_button)
        
        # Set widget properties
        self.setFixedHeight(layout.sizeHint().height() + 10)
        self.setAutoFillBackground(True)
        
        # Default background color
        self.setStyleSheet("background-color: #f5f5f5; border-radius: 6px;")
    
    def run_command(self):
        """Run the command from recent history"""
        # Look for the plugin and command
        for plugin in self.store.plugins:
            if plugin.id == self.recent.plugin_id:
                for cmd in plugin.commands:
                    if cmd.js_function == self.recent.command_function:
                        self.store.execute_command(plugin, cmd)
                        return
        
        # If plugin/command not found, try direct URL
        plugin_id = self.recent.plugin_id.replace(" ", "%20")
        command_name = self.recent.command_name.replace(" ", "%20")
        url = f"noteplan://x-callback-url/runPlugin?pluginID={plugin_id}&command={command_name}"
        import os
        os.system(f"open '{url}'")


class RecentCommandsView(QWidget):
    """View displaying recently used commands"""
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.store = PluginStore.get_instance()
        
        # Create layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        
        # Scroll area
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        
        # Container
        self.container = QWidget()
        self.container_layout = QVBoxLayout(self.container)
        self.container_layout.setContentsMargins(20, 20, 20, 20)
        self.container_layout.setSpacing(10)
        self.container_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        
        # Empty state label
        self.empty_label = QLabel("No recent commands")
        self.empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.empty_label.setStyleSheet("font-size: 18px; color: #666; margin: 40px;")
        self.container_layout.addWidget(self.empty_label)
        
        self.scroll_area.setWidget(self.container)
        layout.addWidget(self.scroll_area)
        
        # Connect signals
        self.store.recents_updated.connect(self.update_recent_list)
        
        # Initialize
        self.update_recent_list()
    
    def update_recent_list(self):
        """Update the list of recent commands"""
        # Clear existing widgets (except empty label)
        for i in reversed(range(self.container_layout.count())):
            item = self.container_layout.itemAt(i)
            if item.widget() != self.empty_label:
                widget = item.widget()
                self.container_layout.removeItem(item)
                if widget:
                    widget.deleteLater()
        
        # Add recent command widgets
        if self.store.recent_commands:
            self.empty_label.setVisible(False)
            for recent in self.store.recent_commands:
                item = RecentCommandItem(recent)
                self.container_layout.addWidget(item)
        else:
            self.empty_label.setVisible(True)
        
        # Add stretch
        self.container_layout.addStretch(1)